﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data.Common;
using System.Configuration;
using System.Data;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Xml.Xsl;
using System.Xml;
using System.IO;

namespace MappingCode
{
    /// <summary>
    /// Interaction logic for MappingPrepare.xaml
    /// </summary>
    public partial class MappingPrepare : Page
    {
        public ObservableCollection<TableInfo> DataSource { get; set; }
        public ObservableCollection<TableInfo> SelectTables { get; set; }

        private DataSet Tables { get; set; }
        private DataSet Data { get; set; }
        public string XslFile { get; set; }
        
        public MappingPrepare()
        {
            InitializeComponent();
            this.DataSource = new ObservableCollection<TableInfo>();
            SelectTables = new ObservableCollection<TableInfo>();
            GetDataTable();
            this.XslFile = System.AppDomain.CurrentDomain.SetupInformation.ApplicationBase + "WPFClass.xsl";
        }

        private void GetDataTable()
        {
            this.Tables = new DataSet();
            string pn = ConfigurationManager.ConnectionStrings[0].ProviderName;
            DbProviderFactory dbfactory = DbProviderFactories.GetFactory(pn);
            var conn = dbfactory.CreateConnection();
            Configuration config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
            conn.ConnectionString = config.AppSettings.Settings["conn"].Value;
            DbCommand cmd = conn.CreateCommand();
            cmd.CommandText = cmd.CommandText = "select * from sysobjects where xtype = 'U' or xtype = 'V' order by name";
            DbDataAdapter dp = dbfactory.CreateDataAdapter();
            dp.SelectCommand = cmd;
            try
            {
                conn.Open();
                //DataSet ds = new DataSet();
                dp.Fill(this.Tables, "table");
                this.FillTree(this.Tables.Tables[0]);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                conn.Close();
            }
        }

        private void GetColumn(string ids)
        {
            string pn = ConfigurationManager.ConnectionStrings[0].ProviderName;
            DbProviderFactory dbfactory = DbProviderFactories.GetFactory(pn);
            var conn = dbfactory.CreateConnection();
            Configuration config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
            conn.ConnectionString = config.AppSettings.Settings["conn"].Value;
            DbCommand cmd = conn.CreateCommand();
            cmd.CommandText = string.Format("select * from syscolumns where id in ({0})" ,ids);
            DbDataAdapter dp = dbfactory.CreateDataAdapter();
            dp.SelectCommand = cmd;
            try
            {
                conn.Open();
                DataSet ds = new DataSet();
                dp.Fill(ds, "column");
                this.Data.Merge(ds);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                conn.Close();
            }            
        }

        private void FillTree(DataTable list)
        {
            TableInfo ti = new TableInfo { IsTable = false, TableName = "用户表" };

            var tabs = new System.Collections.ObjectModel.ObservableCollection<TableInfo>();
            foreach (DataRow item in list.Rows)
            {
                tabs.Add(new TableInfo { IsTable = true, ID = item.Field<int>("id"), TableName = item.Field<string>("name") });
            };
            ti.Child = tabs;
            this.DataSource.Add(ti);
            this.DataContext = this;
        }

        private void CollectData()
        {
            this.Data = new DataSet();
            var query = from row in this.Tables.Tables[0].AsEnumerable()
                        where this.SelectTables.Any(o => o.TableName == row.Field<string>("name"))
                        select row;
            this.Data.Merge(query.ToArray());
            foreach (var item in query)
            {
                GetColumn(item.Field<int>("id").ToString());
            }

        }

        private void btnUnSelct_Click(object sender, RoutedEventArgs e)
        {
            this.SelectTables.Clear();
        }

        private void btnSelct_Click(object sender, RoutedEventArgs e)
        {
            var ti = (TableInfo)this.trvTable.SelectedItem;
            if (ti == null)
                return;
            var list = this.lstSels.ItemsSource as ObservableCollection<TableInfo>;
            if (list != null)
            {
                if (!list.Any(o => o.TableName.Equals(ti.TableName)))
                {
                    list.Add(ti);
                }

            }
        }

        private void btnCopyToClip_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                CollectData();
                XslCompiledTransform xslt = new XslCompiledTransform();
                //string cds = System.AppDomain.CurrentDomain.SetupInformation.ApplicationBase + "WPFClass.xsl";
                xslt.Load(this.XslFile, new XsltSettings(true, true), null);
                //System.IO.FileStream fs = new System.IO.FileStream(this.txtCsFile.Text, System.IO.FileMode.OpenOrCreate);
                System.IO.MemoryStream fs = new System.IO.MemoryStream();
                XmlDocument doc = new XmlDocument();
                doc.LoadXml(this.Data.GetXml());
                //if (fs.Length > 0)
                //    fs.SetLength(0);
                xslt.Transform(doc.CreateNavigator(), null, fs);
                fs.Position = 0;
                StreamReader sr = new StreamReader(fs);
                Clipboard.SetData(DataFormats.Text, sr.ReadToEnd());
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            //MessageBox.Show((string)Clipboard.GetData(DataFormats.Text));
        }

        private void btnFileOpen_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.DefaultExt = ".xsl";
            dlg.Filter = "xsl文件 (.xsl)|*.xsl";
            var result = dlg.ShowDialog();
            if (result ?? false)
            {
                this.XslFile = dlg.FileName;
                this.txtXls.Text = this.XslFile;
            }
        }

        private void btnRefresh_Click(object sender, RoutedEventArgs e)
        {
            GetDataTable();
        }
    }

    public class TableInfo : INotifyPropertyChanged
    {
        public bool IsTable { get; set; }
        public int ID { get; set; }
        private string _tablename;
        public string TableName
        {
            get { return _tablename; }
            set { _tablename = value; OnPropertyChanged(new PropertyChangedEventArgs("TableName")); }
        }

        System.Collections.ObjectModel.ObservableCollection<TableInfo> child;
        public System.Collections.ObjectModel.ObservableCollection<TableInfo> Child
        {
            get { return child; }
            set
            {
                if (child != value)
                {
                    child = value;
                    OnPropertyChanged(new PropertyChangedEventArgs("Child"));
                }
            }
        }

        public TableInfo()
        {
        }

        public event PropertyChangedEventHandler PropertyChanged;
        public void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, e);
            }
        }
    }
}
