<xsl:stylesheet  version="1.0" 
                 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                xmlns:user="http://www.lenhen.com/scripts"
                xmlns="http://www.lenhen.com/codemart">
  <xsl:output omit-xml-declaration="yes" method="text" indent="yes"/>
  <msxsl:script language="C#" implements-prefix="user">
    <![CDATA[
    
        public string GetDealedName(string name)
        {
            if (name == null || name == "")
                return "";
            if(!name.Contains("_"))
            {
                return name;
            }
            string[] ss = name.Split(new char[]{'_'});

            string result = "";
            for (int i = 0; i < ss.Length; i++)
            {
                result += ss[i].Substring(0,1).ToUpper() + ss[i].Substring(1, ss[i].Length - 1).ToLower();
            }
            return result;
        }
        
        public string GetDataType(int i)
        {
            string result = "";
            switch(i)
            {
                //image
                case	34	:	result = "byte[]";	break;
                //text               
                case	35	:	result = "string";	break;
                //Guid
                case  36  : result = "Guid"; break;
                //tinyint
                case	48	:	result = "int";	break;
                //smallint
                case	52	:	result = "int";	break;
                //int
                case	56	:	result = "int";		break;
                //smalldatetime
                case	58	:	result = "DateTime";	break;
                //real
                case	59	:	result = "decimal";	break;
                //money
                case	60	:	result = "decimal";	break;
                //datetime
                case	61	:	result = "DateTime";	break;
                //float
                case	62	:	result = "decimal";	break;
                //ntext
                case	99	:	result = "string";	break;
                //bit
                case	104	:	result = "bool";	break;
                //decimal
                case	106	:	result = "decimal";	break;
                //numeric
                case	108	:	result = "decimal";	break;
                //smallmoney
                case	122	:	result = "decimal";	break;
                //bigint
                case	127	:	result = "int";		break;
                //varbinary
                case	165	:	result = "byte[]";	break;
                //varchar
                case	167	:	result = "string";	break;
                //binary
                case	173	:	result = "byte[]";	break;
                //char
                case	175	:	result = "string";	break;
                //timestamp
                case	189	:	result = "byte[]";	break;
                //nvarchar
                case	231	:	result = "string";	break;
                //nchar
                case	239	:	result = "string";	break;
            }
            return result;        
        }
	]]>
  </msxsl:script>  
  
<xsl:template match="/">
  using System;
  using System.Collections.Generic;
  using System.Linq;
  using System.Text;
  using Wcdf;
  using System.ComponentModel.DataAnnotations;

  namespace
  {
  <xsl:for-each select="//table">
    [Serializable]
    public class <xsl:value-of select="user:GetDealedName(name)"/> : BusinessBase&lt;<xsl:value-of select="user:GetDealedName(name)"/>&gt;
    {
       #region MapperName
       public override string GetDataObjectName()
       {
          return "<xsl:value-of select="name" />";
    }
    #endregion

    protected override void AddBusinessRules()
    {
       base.AddBusinessRules();
    }

    #region Data Feild Property
    <xsl:call-template name="col">
          <xsl:with-param name="varID" select="id"/>
          <xsl:with-param name="varTbname" select="name"/>
       </xsl:call-template>
    #endregion
    }

    [Serializable]
    public class <xsl:value-of select="user:GetDealedName(name)"/>List : BusinessListBase&lt;<xsl:value-of select="user:GetDealedName(name)"/>List,<xsl:value-of select="user:GetDealedName(name)"/>&gt;
    {
      public <xsl:value-of select="user:GetDealedName(name)"/>List()
      {
      }
    }
</xsl:for-each>
  }
</xsl:template>

<xsl:template name="col">
  <xsl:param name="varID"/>
  <xsl:param name="varTbname"/>  
  <xsl:for-each select="//column[id=$varID]">
    public static readonly PropertyInfo&lt;<xsl:value-of select="user:GetDataType(xtype)"/>&gt; <xsl:value-of select="user:GetDealedName(name)"/>Property = RegisterProperty&lt;<xsl:value-of select="user:GetDataType(xtype)"/>&gt;(c => c.<xsl:value-of select="name"/>, "<xsl:value-of select="name"/>");
    public <xsl:value-of select="user:GetDataType(xtype)"/><xsl:text> </xsl:text><xsl:value-of select="user:GetDealedName(name)"/>
    {
        get { return GetProperty(<xsl:value-of select="user:GetDealedName(name)"/>Property); }
        set { SetProperty(<xsl:value-of select="user:GetDealedName(name)"/>Property, value); }
    }
  </xsl:for-each>
</xsl:template>
</xsl:stylesheet>
