﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Wcdf;
using System.ComponentModel.DataAnnotations;

namespace Wcdf.BaseData.Model
{
    [Serializable]
    public class BsUnit : BusinessBase<BsUnit>
    {
        public BsUnit()
        {
            this.KeyID = Guid.NewGuid();
        }

        #region MapperName
        public override string GetDataObjectName()
        {
            return "bs_unit";
        }
        #endregion

        protected override void AddBusinessRules()
        {
            base.AddBusinessRules();
            BusinessRules.AddRule(new Wcdf.Rules.CommonRules.Required(UnitCodeProperty, "单位编码必输"));
            BusinessRules.AddRule(new Wcdf.Rules.CommonRules.Required(UnitNameProperty, "单位名称不能为空"));
        }

        #region Data Feild Property

        public static readonly PropertyInfo<Guid> KeyIDProperty = RegisterProperty<Guid>(c => c.KeyID, "KeyID",RelationshipTypes.PrimaryKey);
        public Guid KeyID
        {
            get { return GetProperty(KeyIDProperty); }
            set { SetProperty(KeyIDProperty, value); }
        }

        public static readonly PropertyInfo<string> UnitCodeProperty = RegisterProperty<string>(c => c.UnitCode, "UnitCode");
        [StringLength(10,ErrorMessage="单位编码不能超过10位")]
        public string UnitCode
        {
            get { return GetProperty(UnitCodeProperty); }
            set { SetProperty(UnitCodeProperty, value); }
        }

        public static readonly PropertyInfo<string> UnitNameProperty = RegisterProperty<string>(c => c.UnitName, "UnitName");
        public string UnitName
        {
            get { return GetProperty(UnitNameProperty); }
            set { SetProperty(UnitNameProperty, value); }
        }

        public static readonly PropertyInfo<string> UnitEnNameProperty = RegisterProperty<string>(c => c.UnitEnName, "UnitEnName");
        public string UnitEnName
        {
            get { return GetProperty(UnitEnNameProperty); }
            set { SetProperty(UnitEnNameProperty, value); }
        }

        public static readonly PropertyInfo<string> ISOUnitProperty = RegisterProperty<string>(c => c.ISOUnit, "ISOUnit");
        public string ISOUnit
        {
            get { return GetProperty(ISOUnitProperty); }
            set { SetProperty(ISOUnitProperty, value); }
        }

        public static readonly PropertyInfo<decimal> ISOFactorsProperty = RegisterProperty<decimal>(c => c.ISOFactors, "ISOFactors");
        public decimal ISOFactors
        {
            get { return GetProperty(ISOFactorsProperty); }
            set { SetProperty(ISOFactorsProperty, value); }
        }

        public static readonly PropertyInfo<string> UnitTypeProperty = RegisterProperty<string>(c => c.UnitType, "UnitType");
        public string UnitType
        {
            get { return GetProperty(UnitTypeProperty); }
            set { SetProperty(UnitTypeProperty, value); }
        }

        public static readonly PropertyInfo<string> IsISOUnitProperty = RegisterProperty<string>(c => c.IsISOUnit, "IsISOUnit");
        public string IsISOUnit
        {
            get { return GetProperty(IsISOUnitProperty); }
            set { SetProperty(IsISOUnitProperty, value); }
        }

        #endregion
    }

    [Serializable]
    public class BsUnitList : BusinessListBase<BsUnitList, BsUnit>
    {
        public BsUnitList()
        {
        }
    }

}
