﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Wcdf.SysProvide.Model;
using System.Xml.Linq;
using System.Configuration;
using System.Collections.ObjectModel;
using Wcdf.Server;

namespace Wcdf.AppClient4WPF.ViewModel
{
    public class LoginViewModel : FrameworkElement
    {        
        public LoginViewModel()
        {
            if (!System.ComponentModel.DesignerProperties.GetIsInDesignMode(this))
            {
                Init();
            }
        }

        public bool Login()
        {
            //this.LoginID
            var result = Wcdf.DataPortal.Invoke<Wcdf.SysProvide.Model.ISysPortal, DataPortalResult>(o => o.UserLogin(this.LoginID, this.Password,this.SelectedOrg.OrgID,this.SelectedOrg.OrgName, null));
            if (result.ReturnObject is Wcdf.Security.WcdfPrincipal)
            {
                //登陆正确
                Wcdf.ApplicationContext.User = result.ReturnObject as Wcdf.Security.WcdfPrincipal;
                Wcdf.ApplicationContext.ContextManager.SetGlobalContext(result.GlobalContext);
                return true;
            }
            else
            {
                MessageBox.Show((string)result.ReturnObject);
                return false;
            }                       
        }

        /// <summary>
        /// 用户认证,设置组织列表
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void Authorization(object sender, Wcdf.Xaml.ExecuteEventArgs e)
        {
            var data = e.MethodParameter as LoginViewModel;
            var pcl = new Wcdf.Security.UnauthenticatedPrincipal();
            var id = pcl.Identity as Wcdf.Security.UnauthenticatedIdentity;
            id.UserName = data.LoginID;
            id.Password = data.Password;
            Wcdf.ApplicationContext.User = pcl;
            try
            {
                var result = Wcdf.DataPortal.Invoke<Wcdf.SysProvide.Model.ISysPortal, SysOrganizationList>(o => o.Authorization(data.LoginID, data.Password));
                this.OrgList = result;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                return;
            }
            if (this.OrgList.Count > 0)
                this.SelectedOrg = this.OrgList[0];
        }
        
        /// <summary>
        /// 初始化界面控件
        /// </summary>
        private void Init()
        {
            this.LoginID = ConfigurationManager.AppSettings["LastLoginID"] ?? string.Empty;
            //配置文件路径.            
            string filePath = string.Concat(AppDomain.CurrentDomain.BaseDirectory, @"AppConfig\UserConfig.xml");

            XElement appServer = XElement.Load(filePath);
            //服务器
            var svrQuery = from nodes in appServer.Element("Server").Elements("Option")
                           select new { Name = nodes.Attribute("name").Value, Value = nodes.Attribute("value").Value };
            this.ServerList = new Dictionary<string, string>(svrQuery.ToDictionary(o=>o.Name,o=>o.Value));
            var svradd = ConfigurationManager.AppSettings["WcdfDataPortalUrl"] ?? string.Empty;
            if (!string.IsNullOrEmpty(svradd))
            {
                this.ServerAddress = this.ServerList.FirstOrDefault(o => o.Value == svradd).Value;
            }
            //语言区域
            var lngQuery = from nodes in appServer.Element("Language").Elements("Option")
                           select new { Name = nodes.Attribute("name").Value, Value = nodes.Attribute("value").Value };
            this.LanguageUI = new Dictionary<string,string>(lngQuery.ToDictionary(o=>o.Name,o=>o.Value));
        }

        public void SaveConfig()
        {

        }

        #region dependency pro

        public static readonly DependencyProperty LoginIDProperty =
        DependencyProperty.Register("LoginID",typeof(string),typeof(LoginViewModel), new PropertyMetadata(null));
        /// <summary>
        /// 用户
        /// </summary>
        public string LoginID
        {
            get { return (string)GetValue(LoginIDProperty); }
            set { SetValue(LoginIDProperty, value); }
        }

        public static readonly DependencyProperty PasswordProperty =
        DependencyProperty.Register("Password", typeof(string), typeof(LoginViewModel), new PropertyMetadata(null));
        /// <summary>
        /// 口令
        /// </summary>
        public string Password
        {
            get { return (string)GetValue(PasswordProperty); }
            set { SetValue(PasswordProperty, value); }
        }

        public static readonly DependencyProperty SelectedOrgProperty =
        DependencyProperty.Register("SelectedOrg", typeof(SysOrganization), typeof(LoginViewModel), new PropertyMetadata(null));
        /// <summary>
        /// 
        /// </summary>
        public SysOrganization SelectedOrg
        {
            get { return (SysOrganization)GetValue(SelectedOrgProperty); }
            set { SetValue(SelectedOrgProperty, value); }
        }

        public static readonly DependencyProperty OrgListProperty =
        DependencyProperty.Register("OrgList", typeof(SysOrganizationList), typeof(LoginViewModel), new PropertyMetadata(null));
        /// <summary>
        /// 
        /// </summary>
        public SysOrganizationList OrgList
        {
            get { return (SysOrganizationList)GetValue(OrgListProperty); }
            set { SetValue(OrgListProperty, value); }
        }

        public static readonly DependencyProperty ServerAddressProperty =
        DependencyProperty.Register("ServerAddress", typeof(string), typeof(LoginViewModel), new PropertyMetadata(null));
        /// <summary>
        /// 服务地址
        /// </summary>
        public string ServerAddress
        {
            get { return (string)GetValue(ServerAddressProperty); }
            set { SetValue(ServerAddressProperty, value); }
        }

        public static readonly DependencyProperty ServerListProperty =
        DependencyProperty.Register("ServerList", typeof(Dictionary<string,string>), typeof(LoginViewModel), new PropertyMetadata(null));
        /// <summary>
        /// 
        /// </summary>
        public Dictionary<string,string> ServerList
        {
            get { return (Dictionary<string,string>)GetValue(ServerListProperty); }
            set { SetValue(ServerListProperty, value); }
        }

        public static readonly DependencyProperty LanguageUIProperty =
        DependencyProperty.Register("Language", typeof(Dictionary<string,string>), typeof(LoginViewModel), new PropertyMetadata(null));
        /// <summary>
        /// 
        /// </summary>
        public Dictionary<string,string> LanguageUI
        {
            get { return (Dictionary<string,string>)GetValue(LanguageUIProperty); }
            set { SetValue(LanguageUIProperty, value); }
        }

        public static readonly DependencyProperty IsPassProperty =
        DependencyProperty.Register("IsPass", typeof(bool), typeof(LoginViewModel), new PropertyMetadata(null));
        /// <summary>
        /// 是否可以登陆
        /// </summary>
        public bool IsPass
        {
            get { return (bool)GetValue(IsPassProperty); }
            set { SetValue(IsPassProperty, value); }
        }
        #endregion
    }
}
