﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Wcdf.Rules;
using System.Text.RegularExpressions;
using System.Data;

namespace Wcdf.Authorization
{
    public class DataPermission : Wcdf.Rules.IDataPermissionProvider
    {

        #region IAuthorizationRule Members


        public Rules.ObjectRight FindObjectRight(string roleID, string objectName, Rules.AuthorizationActions action,List<ObjectRight> source)
        {
            bool isSelect = action == AuthorizationActions.ReadProperty;
            
            var user = ApplicationContext.User as Wcdf.Security.WcdfPrincipal;
            //var agent = new List<SharpDevelop.Security.UserInfo>(ApplicationContext.Current.GlobalContext["Agent"] as SharpDevelop.Security.UserInfo[]);
            var agent = user.Agent;
            if ((null == agent || agent.Count == 0) && source.Count == 1)
            {
                return source[0];
            }
            //大于1时或者存在代理用户.需要生成新对象.
            ObjectRight result = source[0].Clone() as ObjectRight;
            if (isSelect)
                AgentCheck(result, agent);
            foreach (ObjectRight or in source)
            {
                if (result.RoleID != or.RoleID)
                {
                    if (isSelect)
                        AgentCheck(or, agent);
                    result.IsCanDelete = result.IsCanDelete || or.IsCanDelete;
                    result.IsCanInsert = result.IsCanInsert || or.IsCanInsert;
                    result.IsCanSelect = result.IsCanSelect || or.IsCanSelect;
                    result.IsCanUpdate = result.IsCanUpdate || or.IsCanUpdate;
                    foreach (FieldRight fr in result.FieldRightList)
                    {
                        FieldRight orfr = or.FieldRightList.Find(s => s.FieldName == fr.FieldName);
                        if (null != orfr)
                        {
                            fr.IsUnnullable = fr.IsUnnullable && orfr.IsUnnullable;
                            fr.IsReadOnly = fr.IsReadOnly && orfr.IsReadOnly;
                            fr.IsUnvisible = fr.IsUnvisible && orfr.IsUnvisible;
                            fr.SelectCondition = string.IsNullOrEmpty(orfr.SelectCondition) ? fr.SelectCondition :
                                string.Format("(({0}) or ({1}))", fr.SelectCondition, orfr.SelectCondition);
                            //fr.SelectCriteria = string.IsNullOrEmpty(orfr.SelectCriteria) ? fr.SelectCriteria :
                            //    string.Format("(({0}) or ({1}))", fr.SelectCriteria, GetRoleFilter(orfr.SelectCriteria, or.RoleID));
                        }
                    }
                }
            }

            return result;
        }

        private void AgentCheck(ObjectRight or, List<Security.UserInfo> agent)
        {
            //var user = ApplicationContext.Current.User.Identity as SharpDevelop.Security.UserIdentity;
            if (null == agent || agent.Count == 0)
                return;
            string result = string.Empty;
            foreach (FieldRight fr in or.FieldRightList)
            {
                if (string.IsNullOrEmpty(fr.SelectCondition))
                    continue;
                string filter = fr.SelectCondition;
                foreach (Security.UserInfo info in agent.FindAll(op => op.RoleID == or.RoleID))
                {
                    filter = string.Format(" {0} or {1} ", filter, ReplaceUserInfo(fr.SelectCondition, info));
                }
                fr.SelectCondition = "(" + filter + ")";
            }
        }

        private string ReplaceUserInfo(string sql, Security.UserInfo info)
        {
            if (string.IsNullOrEmpty(sql))
                return sql;
            if (sql.IndexOf("@UserID", StringComparison.OrdinalIgnoreCase) > 0)
            {
                sql = Regex.Replace(sql, "@UserID", info.UserID, RegexOptions.IgnoreCase);
            }
            if (sql.IndexOf("@CorpOrgID", StringComparison.OrdinalIgnoreCase) > 0)
            {
                sql = Regex.Replace(sql, "@CorpOrgID", info.CorpOrgID, RegexOptions.IgnoreCase);
            }
            if (sql.IndexOf("@OrgID", StringComparison.OrdinalIgnoreCase) > 0)
            {
                sql = Regex.Replace(sql, "@OrgID", info.OrgID, RegexOptions.IgnoreCase);
            }
            return sql;
        }

        /// <summary>
        /// 从数据库设置中取得权限
        /// </summary>
        /// <param name="roleID"></param>
        /// <param name="objectName"></param>
        /// <returns></returns>
        public List<ObjectRight> GetObjectRight(string userID, string objectName)
        {
            List<ObjectRight> result = new List<ObjectRight>();
            Data.DataAccess dal = new Data.DataAccess();
            StringBuilder sb = new StringBuilder();
            //sb.Append("select sys_role_table_right.KeyID,sys_role_table_right.RoleID, sys_table.ObjectName,IsCanSelect,IsCanInsert,IsCanUpdate,IsCanDelete");
            //sb.Append("  from sys_role_table_right,sys_table,sys_role_user");
            //sb.Append(" where sys_role_table_right.TableKeyID = sys_table.KeyID");
            //sb.Append("   and sys_role_user.RoleID = sys_role_table_right.RoleID");
            //取得用户的对应权限对象
            sb.AppendFormat("select * from sys_vm_user_object where");
            sb.AppendFormat("   and UserID = '{0}'", userID);
            sb.AppendFormat("   and ObjectName = '{0}'", objectName);
            string trKeyID = string.Empty;
            DataSet ds = dal.ExecuteDataSet(sb.ToString());
            if (ds.Tables[0].Rows.Count != 0)
            {
                foreach (DataRow row in ds.Tables[0].Rows)
                {
                    trKeyID = row["KeyID"].ToString();
                    ObjectRight newor = new ObjectRight()
                    {
                        RoleID = row["RoleID"].ToString(),
                        ObjectName = row["ObjectName"].ToString(),
                        IsCanSelect = "T" == row["IsCanSelect"].ToString(),
                        IsCanInsert = "T" == row["IsCanInsert"].ToString(),
                        IsCanUpdate = "T" == row["IsCanUpdate"].ToString(),
                        IsCanDelete = "T" == row["IsCanDelete"].ToString()
                    };
                    sb = new StringBuilder();
                    sb.Append("select * ");
                    sb.Append("  from sys_vm_user_object");
                    sb.AppendFormat("  where TableRightKeyID = '{0}'", trKeyID);
                    DataSet fields = dal.ExecuteDataSet(sb.ToString());
                    foreach (DataRow frow in fields.Tables[0].Rows)
                    {
                        newor.FieldRightList.Add(new FieldRight()
                        {
                            FieldName = frow["FieldName"].ToString(),
                            IsReadOnly = "T".Equals(frow["IsReadOnly"]),
                            IsUnnullable = "T".Equals(frow["IsUnnullable"]),
                            IsUnvisible = "T".Equals(frow["IsUnvisible"]),
                            SelectCondition = frow.IsNull("SelectCondition") ? string.Empty : frow["SelectCondition"].ToString()
                        });
                    }
                    result.Add(newor);
                }
            }
            return result;
        }

        public ObjectRight EmptyObjectRight(string roleID, string objectName)
        {
            ObjectRight noctrl = new ObjectRight()
            {
                RoleID = roleID,
                ObjectName = objectName,
                IsControl = false,
            };
            return noctrl;
        }

        public List<ObjectRight> GetList(string roleID, string objectName, List<ObjectRight> source)
        {
            return source.FindAll(o =>
                        ("," + roleID + ",").IndexOf("," + o.RoleID + ",") > -1 &&
                        o.ObjectName.Equals(objectName, StringComparison.OrdinalIgnoreCase));
        }

        public bool Compare(ObjectRight or1, ObjectRight or2)
        {
            if (or1.RoleID == or2.RoleID && or1.ObjectName == or2.ObjectName)
                return true;
            return false;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="userName"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        public bool Validate(string userName, string password)
        {
            var da = new Wcdf.Data.DataAccess();
            string sql = string.Format("select 1 from sys_user where userid ='{0}' and password = '{1}'", userName, password);
            object val = da.ExecuteScalar(sql);
            if (val == null)
            {
                throw new Exception("用户名或密码错误！");
            }
            return true;
        }
        #endregion
    }
}
