﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.Specialized;
using System.Xml.Linq;
using System.Collections.ObjectModel;

namespace Wcdf.Control.LookUp
{
    /// <summary>
    /// 需要单件模式
    /// </summary>
    internal sealed class LookUpConfig
    {
        private Dictionary<string, ColumnInfoCollection> InnerConfigs { get; set; }

        private LookUpConfig()
        {
            //MethodInfo myMethod = myString.GetType().GetMethods().First(m => m.Name.Equals("Show") && m.IsGenericMethod);
            //myMethod.MakeGenericMethod(typeof(int)).Invoke(myString, new object[] { 100 });
            InnerConfigs = new Dictionary<string, ColumnInfoCollection>();
            LoadConfig();
        }

        public static readonly LookUpConfig Configs = new LookUpConfig();

        private void LoadConfig()
        {
            //配置文件路径.
            string filePath = string.Concat(AppDomain.CurrentDomain.BaseDirectory, @"AppConfig\LookUp.xml");
            if (!System.IO.File.Exists(filePath))
            {
                throw new System.IO.FileNotFoundException("程序文件缺失:LookUp.Config配置文件不存在!");
            }
            //解释配置文件
            XElement appServer = XElement.Load(filePath);
            var configQuery = from configs in appServer.Elements("LookUp")
                              select configs;
            foreach (XElement ele in configQuery)
            {
                ColumnInfoCollection infos = new ColumnInfoCollection();
                if (ele.Attribute("Name") == null)
                    continue;
                infos.Key = ele.Attribute("Name").Value;
                infos.CriteriaSql = ele.Attribute("CriteriaSql") == null ? null : ele.Attribute("CriteriaSql").Value;
                infos.DataEntityClass = ele.Attribute("DataEntityClass") == null ? null : ele.Attribute("DataEntityClass").Value;
                infos.DisplayMember = ele.Attribute("DisplayMember") == null ? null : ele.Attribute("DisplayMember").Value;
                infos.ValueMember = ele.Attribute("ValueMember") == null ? null : ele.Attribute("ValueMember").Value;
                infos.Height = ele.Attribute("Height") == null ? 0 : int.Parse(ele.Attribute("Height").Value);
                infos.Width = ele.Attribute("Width") == null ? 0 : int.Parse(ele.Attribute("Width").Value);
                var child = from ch in ele.Elements("Column")
                            select ch;
                List<string> fields = new List<string>();
                foreach (XElement col in child)
                {
                    if (col.Attribute("FieldName") == null || string.IsNullOrEmpty(col.Attribute("FieldName").Value))
                        continue;
                    infos.Add(new ColumnInfo
                    {
                        Caption = col.Attribute("Caption") == null ? null : col.Attribute("Caption").Value,
                        FieldName = col.Attribute("FieldName") == null ? null : col.Attribute("FieldName").Value,
                        FormatString = col.Attribute("FormatString") == null ? null : col.Attribute("FormatString").Value,
                        Visible = col.Attribute("Visible") == null ? true : !string.IsNullOrEmpty(col.Attribute("Visible").Value),
                        Width = col.Attribute("Width") == null ? 0 : int.Parse(col.Attribute("Width").Value),
                    }
                    );
                    fields.Add(col.Attribute("FieldName").Value);
                }
                infos.Fields = fields.ToArray();
                if (!InnerConfigs.Keys.Contains(infos.Key))
                    InnerConfigs.Add(infos.Key, infos);
            }
            //select new { Name = nodes.Attribute("name").Value, Value = nodes.Attribute("value").Value };
        }
        /// <summary>
        /// 列信息
        /// </summary>
        /// <param name="configName"></param>
        /// <returns></returns>
        internal ColumnInfoCollection this[string configName]
        {
            get
            {
                if (!this.InnerConfigs.Keys.Contains(configName))
                {
                    return null;
                }
                else
                {
                    return InnerConfigs[configName];
                }
            }
        }

        /// <summary>
        /// 列信息
        /// </summary>      
    }

    internal class ColumnInfoCollection : Collection<ColumnInfo>
    {
        public string DataEntityClass { get; set; }
        public string CriteriaSql { get; set; }
        public string DisplayMember { get; set; }
        public string ValueMember { get; set; }
        public string Key { get; set; }
        public int Height { get; set; }
        public int Width{ get; set; }
        public string[] Fields { get; set; }
    }

    internal class ColumnInfo
    {
        //标题
        public string Caption { get; set; }
        //字段名
        public string FieldName { get; set; }
        //格式
        public string FormatString { get; set; }
        //可见
        public bool Visible { get; set; }
        //长度
        public int Width { get; set; }
    }
}
