﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.ComponentModel;
using DevExpress.Xpf.Grid.LookUp;
using Wcdf.Data;
using System.Collections;
using Wcdf.Control.LookUp;
using DevExpress.Xpf.Grid;
using System.Windows;

namespace Wcdf.Control
{
    public class LookUpEditEx : LookUpEdit
    {
        #region UI Property
        public static DependencyProperty DataHandleConfigNameProperty = DependencyProperty.Register("DataHandleConfigName",
                                                                                              typeof(string),
                                                                                              typeof(LookUpEditEx),
                                                                                              new PropertyMetadata(null));

        public string DataHandleConfigName
        {
            get { return (string)GetValue(DataHandleConfigNameProperty); }
            set { SetValue(DataHandleConfigNameProperty, value); }
        }
        #endregion

        /// <summary>
        /// 是否读取数据
        /// </summary>
        private bool IsColumnsCreated
        {
            get;
            set;
        }

        public LookUpEditEx()
            : base()
        {
            this.AutoPopulateColumns = false;
            this.IsPopupAutoWidth = true;
            this.PopupMinHeight = 100;
            this.PopupMinWidth = this.Width;
            this.Initialized += new EventHandler(LookUpEditEx_Initialized);
        }

        void LookUpEditEx_Initialized(object sender, EventArgs e)
        {
            if (!System.ComponentModel.DesignerProperties.GetIsInDesignMode(this))
            {
                InitDropDown();
            }
        }

        /// <summary>
        /// 初始化下拉菜单.
        /// </summary>
        internal void InitDropDown()
        {
            try
            {
                if (string.IsNullOrEmpty(this.DataHandleConfigName))
                    return;
                var cols = Wcdf.Control.LookUp.LookUpConfig.Configs[this.DataHandleConfigName];
                if (null != cols)
                {
                    Type dataEntity = Type.GetType(cols.DataEntityClass);
                    Criteria filter = new Criteria();
                    filter.AddFilter(cols.CriteriaSql);
                    filter.Columns = cols.Fields;
                    var source = Wcdf.DataPortal.Fetch(dataEntity, filter) as IEnumerable;
                    this.ItemsSource = source;
                    this.DisplayMember = cols.DisplayMember;
                    this.ValueMember = cols.ValueMember;
                    this.PopupHeight = (cols.Height == 0 ? this.PopupMinHeight : cols.Height); ;
                    this.PopupWidth = (cols.Width == 0 ? this.Width : cols.Width);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("无法访问所选择的服务." + ex.Message);
            }
        }

        private void CreateDataControl()
        {
            var dc = this.GetGridControl();
            if (dc == null || this.IsColumnsCreated)
                return;
            var cols = Wcdf.Control.LookUp.LookUpConfig.Configs[this.DataHandleConfigName];
            foreach (ColumnInfo col in cols)
            {
                GridColumn info = new GridColumn();
                info.Header = col.Caption;
                info.Visible = col.Visible;
                //info.Visible
                if (string.IsNullOrEmpty(col.FieldName))
                    return;
                info.FieldName = col.FieldName;
                if (col.Width != 0)
                    info.Width = col.Width;
                dc.Columns.Add(info);
            }
            this.IsColumnsCreated = true;
        }

        protected override void OnPopupOpened()
        {
            base.OnPopupOpened();
            CreateDataControl();
        }

    }
}
