﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Wcdf.Xaml
{
    /// <summary>
    /// 基于业务的MVVM模式类的基本数据操作方法调用
    /// </summary>
    /// <typeparam name="T">Type of the Model object.</typeparam>
    public abstract class ViewModel<T> : ViewModelBase<T>
    {
        #region Verbs

        /// <summary>
        /// Saves the Model, first committing changes
        /// if ManagedObjectLifetime is true.
        /// </summary>
        public virtual void Save(object sender, ExecuteEventArgs e)
        {
            BindControlEndEdit(e);
            BeginSave();
        }

        /// <summary>
        /// Cancels changes made to the model 
        /// if ManagedObjectLifetime is true.
        /// </summary>
        public virtual void Cancel(object sender, ExecuteEventArgs e)
        {
            //BindControlEndEdit(e);
            DoCancel();
        }

        /// <summary>
        /// Adds a new item to the Model (if it
        /// is a collection).
        /// </summary>
        public virtual void AddNew(object sender, ExecuteEventArgs e)
        {
#if SILVERLIGHT
      BeginAddNew();
#else
            DoAddNew();
#endif
        }

        /// <summary>
        /// Removes an item from the Model (if it
        /// is a collection).
        /// </summary>
        public virtual void Remove(object sender, ExecuteEventArgs e)
        {
            //BindControlEndEdit(e);
            DoRemove(e.MethodParameter);
        }

        /// <summary>
        /// Marks the Model for deletion (if it is an
        /// editable root object).
        /// </summary>
        public virtual void Delete(object sender, ExecuteEventArgs e)
        {
            DoDelete();
        }

        #endregion

        private void BindControlEndEdit(ExecuteEventArgs e)
        {
            //焦点转移,以触发控件级的数据绑定的EndEdit方法.
            if ((e.TriggerSource).Focusable)
                (e.TriggerSource).Focus();
        }
    }
}
