﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Wcdf.Xaml;
using Wcdf.SysProvide.Model;
using System.Windows;
using Wcdf.ToolClass;

namespace Wcdf.SysProvide.Client.ViewModel
{
    public class SysCodeTypeListVM : ViewModel<SysCodeTypeList>
    {

        public SysCodeTypeListVM()
        {

            if (!System.ComponentModel.DesignerProperties.GetIsInDesignMode(this))
            {
                this.ManageObjectLifetime = false;
                DoRefresh(() => Wcdf.DataPortal.Fetch<SysCodeTypeList>());
                this.TreeMode = new EntityTree<SysCodeTypeList, SysCodeType>(this.Model, new SysCodeType(), "KeyID", "ParentKeyID");
            }
        }

        public void AddNewClass(object sender, Wcdf.Xaml.ExecuteEventArgs e)
        {
            var parent = e.MethodParameter as EntityTree<SysCodeTypeList, SysCodeType>;
            if (parent.Node.IsClass != "1")
            {
                MessageBox.Show("非目录下不可建分类");
                return;
            }
            var child = new EntityTree<SysCodeTypeList, SysCodeType>();
            child.Node = Wcdf.DataPortal.Create<SysCodeType>();
            child.Node.KeyID = Guid.NewGuid();
            child.Node.IsClass = "1";
            DlgCodeType dlg = new DlgCodeType(child.Node);
            dlg.ShowDialog();
            if (dlg.DialogResult ?? false)
            {
                child.Node.ParentKeyID = parent.Node.KeyID.ToString();
                child.Node.Save();
                parent.AddNewItem(child);
                //this.DoSave();
            }
        }

        public void AddNewType(object sender, Wcdf.Xaml.ExecuteEventArgs e)
        {
            var parent = e.MethodParameter as EntityTree<SysCodeTypeList, SysCodeType>;
            if (parent.Node.IsClass != "1")
            {
                MessageBox.Show("类型下不可建子级");
                return;
            }
            var child = new EntityTree<SysCodeTypeList, SysCodeType>();
            child.Node = Wcdf.DataPortal.Create<SysCodeType>();
            child.Node.KeyID = Guid.NewGuid();
            child.Node.IsClass = "0";
            DlgCodeType dlg = new DlgCodeType(child.Node);
            dlg.ShowDialog();
            if (dlg.DialogResult ?? false)
            {
                child.Node.ParentKeyID = parent.Node.KeyID.ToString();
                child.Node.Save();
                parent.AddNewItem(child);
            }
        }

        public void EditType(object sender, Wcdf.Xaml.ExecuteEventArgs e)
        {
            var parent = e.MethodParameter as EntityTree<SysCodeTypeList, SysCodeType>;
            DlgCodeType dlg = new DlgCodeType(parent.Node);
            dlg.ShowDialog();
            if (dlg.DialogResult ?? false)
            {
                //this.BeginSave();
                parent.Node.Save();
            }
        }

        public void DelType(object sender, Wcdf.Xaml.ExecuteEventArgs e)
        {
            var curItem = e.MethodParameter as EntityTree<SysCodeTypeList, SysCodeType>;
            if (curItem.ChildNodes != null && curItem.ChildNodes.Count != 0)
            {
                MessageBox.Show("包含下级目录,不能直接删除.");
                return;
            }
            var item = curItem.Node;            
            this.Model.Remove(item);
            this.BeginSave();
            curItem.Parent.ChildNodes.Remove(curItem);
        }
        #region UI
        public static readonly DependencyProperty TreeModeProperty =
        DependencyProperty.Register("TreeMode", typeof(EntityTree<SysCodeTypeList, SysCodeType>),
        typeof(SysCodeTypeListVM), new PropertyMetadata(null));
        /// <summary>
        /// 
        /// </summary>
        public EntityTree<SysCodeTypeList, SysCodeType> TreeMode
        {
            get { return (EntityTree<SysCodeTypeList, SysCodeType>)GetValue(TreeModeProperty); }
            set { SetValue(TreeModeProperty, value); }
        }

        public void ShowItem(object sender, Wcdf.Xaml.ExecuteEventArgs e)
        {

        }
        #endregion
    }
}
