﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Collections.Specialized;

namespace Wcdf.Core
{
    /// <summary>
    /// 包含了子对象更改的事件
    /// </summary>
    public class ChildChangedEventArgs : EventArgs
    {
        /// <summary>
        /// 更改的子对象引用
        /// </summary>
        public object ChildObject { get; private set; }
        /// <summary>
        /// Gets the PropertyChangedEventArgs object from the
        /// child's PropertyChanged event, if the child is
        /// not a collection or list.
        /// </summary>
        public PropertyChangedEventArgs PropertyChangedArgs { get; private set; }
        /// <summary>
        /// 如果子对象是ObservableCollection类型.则当子对象引发CollectionChanged事件时.
        /// 会返回NotifyCollectionChangedEventArgs对象.        
        /// </summary>
        public NotifyCollectionChangedEventArgs CollectionChangedArgs { get; private set; }

#if !SILVERLIGHT
        /// <summary>
        /// Gets the ListChangedEventArgs object from the
        /// child's ListChanged event, if the child is a
        /// collection or list.
        /// </summary>
        public ListChangedEventArgs ListChangedArgs { get; private set; }

        /// <summary>
        /// Creates an instance of the object.
        /// </summary>
        /// <param name="childObject">
        /// Reference to the child object that was changed.
        /// </param>
        /// <param name="listArgs">
        /// ListChangedEventArgs object or null.
        /// </param>
        /// <param name="propertyArgs">
        /// PropertyChangedEventArgs object or null.
        /// </param>
        public ChildChangedEventArgs(object childObject, PropertyChangedEventArgs propertyArgs, ListChangedEventArgs listArgs)
            : this(childObject, propertyArgs)
        {
            this.ListChangedArgs = listArgs;
        }
#endif

        /// <summary>
        /// Creates an instance of the object.
        /// </summary>
        /// <param name="childObject">
        /// Reference to the child object that was changed.
        /// </param>
        /// <param name="propertyArgs">
        /// PropertyChangedEventArgs object or null.
        /// </param>
        public ChildChangedEventArgs(object childObject, PropertyChangedEventArgs propertyArgs)
        {
            this.ChildObject = childObject;
            this.PropertyChangedArgs = propertyArgs;
        }

        /// <summary>
        /// Creates an instance of the object.
        /// </summary>
        /// <param name="childObject">
        /// Reference to the child object that was changed.
        /// </param>
        /// <param name="listArgs">
        /// ListChangedEventArgs object or null.
        /// </param>
        /// <param name="propertyArgs">
        /// PropertyChangedEventArgs object or null.
        /// </param>
        public ChildChangedEventArgs(object childObject, PropertyChangedEventArgs propertyArgs, NotifyCollectionChangedEventArgs listArgs)
            : this(childObject, propertyArgs)
        {
            this.CollectionChangedArgs = listArgs;
        }
    }
}
