﻿using System;
using System.Collections.Specialized;
using Wcdf.Serialization.Mobile;

namespace Wcdf.Core
{
    /// <summary>
    /// 可通过MobileFormatter序列化的字典类型.
    /// </summary>
    [Serializable()]
    public class ContextDictionary : HybridDictionary, IMobileObject
    {
        #region IMobileObject Members
        
        void IMobileObject.GetState(SerializationInfo info)
        {
            foreach (string key in this.Keys)
            {
                object value = this[key];
                if (!(value is IMobileObject))
                    info.AddValue(key, value);
            }
        }

        void IMobileObject.GetChildren(SerializationInfo info, MobileFormatter formatter)
        {
            foreach (string key in this.Keys)
            {
                object value = this[key];
                IMobileObject mobile = value as IMobileObject;
                if (mobile != null)
                {
                    SerializationInfo si = formatter.SerializeObject(mobile);
                    info.AddChild(key, si.ReferenceId);
                }
            }
        }

        void IMobileObject.SetState(SerializationInfo info)
        {
            foreach (string key in info.Values.Keys)
            {
                Add(key, info.Values[key].Value);
            }
        }

        void IMobileObject.SetChildren(SerializationInfo info, MobileFormatter formatter)
        {
            foreach (string key in info.Children.Keys)
            {
                int referenceId = info.Children[key].ReferenceId;
                this.Add(key, formatter.GetObject(referenceId));
            }
        }

        #endregion
    }
}
