﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Wcdf.Core
{
    /// <summary>
    /// 定义框架可编辑列表对象的通用方法    
    /// </summary>
    /// <remarks>
    /// It is strongly recommended that the implementations
    /// of the methods in this interface be made Private
    /// so as to not clutter up the native interface of
    /// the collection objects.
    /// </remarks>
    public interface IEditableCollection : IBusinessObject, ISupportUndo, ITrackStatus
    {
        /// <summary>
        /// 将子对象加入到当前列表中.
        /// </summary>
        /// <param name="child"></param>
        /// <remarks>该操作只是纯碎的列表操作.被未将子对象进行业务校验.
        /// </remarks>
        void AddChild(Core.IEditableBusinessObject child);
        /// <summary>
        /// Removes the specified child from the parent
        /// collection.
        /// </summary>
        /// <param name="child">Child object to be removed.</param>
        void RemoveChild(Core.IEditableBusinessObject child);
        /// <summary>
        /// 当对象做为子对象时.设置它的父对象        
        /// </summary>
        /// <param name="parent">A reference to the parent collection object.</param>
        void SetParent(IParent parent);
        /// <summary>
        /// 取得删除的项列表.
        /// </summary>
        object GetDeletedList();
    }
}
