﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Wcdf.Core
{
    /// <summary>
    /// 定义对象工厂接口以创建IPropertyInfo属性.    
    /// </summary>
    public interface IPropertyInfoFactory
    {
        /// <summary>
        /// Creates a new instance of PropertyInfo.
        /// </summary>
        /// <param name="containingType">
        /// Type of business class that contains the property
        /// declaration.
        /// </param>
        /// <param name="name">Name of the property.</param>
        Wcdf.PropertyInfo<T> Create<T>(Type containingType, string name);
        /// <summary>
        /// Creates a new instance of PropertyInfo.
        /// </summary>
        /// <param name="containingType">
        /// Type of business class that contains the property
        /// declaration.
        /// </param>
        /// <param name="name">Name of the property.</param>
        /// <param name="friendlyName">
        /// Friendly display name for the property.
        /// </param>
        Wcdf.PropertyInfo<T> Create<T>(Type containingType, string name, string friendlyName);
        /// <summary>
        /// Creates a new instance of PropertyInfo.
        /// </summary>
        /// <param name="containingType">
        /// Type of business class that contains the property
        /// declaration.
        /// </param>
        /// <param name="name">Name of the property.</param>
        /// <param name="friendlyName">
        /// Friendly display name for the property.
        /// </param>
        /// <param name="relationship">Relationship with
        /// referenced object.</param>
        Wcdf.PropertyInfo<T> Create<T>(Type containingType, string name, string friendlyName, RelationshipTypes relationship);
        /// <summary>
        /// Creates a new instance of PropertyInfo.
        /// </summary>
        /// <param name="containingType">
        /// Type of business class that contains the property
        /// declaration.
        /// </param>
        /// <param name="name">Name of the property.</param>
        /// <param name="friendlyName">
        /// Friendly display name for the property.
        /// </param>
        /// <param name="defaultValue">
        /// Default value for the property.
        /// </param>
        Wcdf.PropertyInfo<T> Create<T>(Type containingType, string name, string friendlyName, T defaultValue);
        /// <summary>
        /// Creates a new instance of PropertyInfo.
        /// </summary>
        /// <param name="containingType">
        /// Type of business class that contains the property
        /// declaration.
        /// </param>
        /// <param name="name">Name of the property.</param>
        /// <param name="friendlyName">
        /// Friendly display name for the property.
        /// </param>
        /// <param name="defaultValue">
        /// Default value for the property.
        /// </param>
        /// <param name="relationship">Relationship with
        /// referenced object.</param>
        Wcdf.PropertyInfo<T> Create<T>(Type containingType, string name, string friendlyName, T defaultValue, RelationshipTypes relationship);
    }
}
