﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Wcdf.Core
{
    /// <summary>
    /// 对象保存自身的一些方法
    /// Specifies that the object can save
    /// itself.
    /// </summary>
    public interface ISavable
    {
        /// <summary>
        /// 保存对象到数据库中.
        /// </summary>
        /// <returns>保存后的对象.</returns>
        object Save();
        /// <summary>
        ///  保存对象到数据库中.
        /// </summary>
        /// <returns>保存后的对象.</returns>
        /// <param name="forceUpdate">是否强制保存</param>
        object Save(bool forceUpdate);
        /// <summary>
        /// Saves the object to the database asynchronously. The saved event will contain
        /// the new object when the save operation completes.
        /// </summary>
        void BeginSave();
        /// <summary>
        /// Saves the object to the database asynchronously. The saved event will contain
        /// the new object when the save operation completes.
        /// </summary>
        /// <param name="userState">
        /// User state object.
        /// </param>
        void BeginSave(object userState);
        /// <summary>
        /// INTERNAL CSLA .NET USE ONLY.
        /// </summary>
        /// <param name="newObject">
        /// 被保存的对象..
        /// </param>
        void SaveComplete(object newObject);
        /// <summary>
        /// 对象保存后事件.
        /// </summary>
        event EventHandler<SavedEventArgs> Saved;
        /// <summary>
        /// 对象保存前事件.
        /// </summary>
        event EventHandler<BeforeSaveEventArgs> BeforeSave;
    }
}
