﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Wcdf.Core
{
    /// <summary>
    /// 定义了在N层撤消中由UI调用的通用方法.    
    /// </summary>
    /// <remarks>
    /// 该接口是为了帮助UI开发人员编辑业务对象而设计.   
    /// 当使用N层撤销的时候，UI应该首先调用BeginEdit().
    /// 如果用户随后点击了取消按钮，就会调用CancelEdit()方法.
    /// 如果用户点击了保存或接受按钮，就会调用ApplyEdit()方法.
    /// </remarks>
    public interface ISupportUndo
    {
        /// <summary>
        /// 启动对象编辑
        /// </summary>
        void BeginEdit();
        /// <summary>
        /// 取消当前修改,将对象快照返回上个版本.
        /// </summary>
        void CancelEdit();
        /// <summary>
        /// 提交当前修改
        /// </summary>
        void ApplyEdit();
    }
}
