﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Wcdf.Core
{
    /// <summary>
    /// 定义了实现N层撤消能力需要的方法.    
    /// </summary>
    /// <remarks>
    /// 该接口由UndoableBase类发起开始,取消,提交编辑操作.  
    /// 接口方法由框架内部调用,在N层撤消功能的UI实现方法中被调用.
    /// </remarks>
    public interface IUndoableObject
    {
        /// <summary>
        /// 取得当前对象层数
        /// </summary>
        int EditLevel { get; }
        /// <summary>
        /// 复制对象到堆栈顶部中.
        /// </summary>
        /// <param name="parentEditLevel">
        /// 父对象的层数
        /// </param>
        /// <param name="parentBindingEdit">
        /// 指示父对象是否处于数据绑定的编辑模式.
        /// </param>
        void CopyState(int parentEditLevel, bool parentBindingEdit);
        /// <summary>
        /// 将对象返回到堆栈中最新的快照        
        /// </summary>
        /// <remarks>
        /// Restores the state of the object to its
        /// previous value by taking the data out of
        /// the stack and restoring it into the fields
        /// of the object.
        /// <param name="parentEditLevel">
        /// 父对象的层数
        /// </param>
        /// <param name="parentBindingEdit">
        /// 指示父对象是否处于数据绑定的编辑模式.
        /// </param>
        void UndoChanges(int parentEditLevel, bool parentBindingEdit);
        /// <summary>
        /// 接受当前对象所做的更改
        /// </summary>
        /// <remarks>
        /// 堆栈中最新的对象快照将移除,这样提交的更改就做用于对象.
        /// </remarks>
        /// <param name="parentEditLevel">
        /// 父对象的层数
        /// </param>
        /// <param name="parentBindingEdit">
        /// 指示父对象是否处于数据绑定的编辑模式.
        /// </param>
        void AcceptChanges(int parentEditLevel, bool parentBindingEdit);
    }
}
