﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Wcdf.Serialization.Mobile;
using Wcdf.Serialization;
using System.ComponentModel;

namespace Wcdf.Core
{
    /// <summary>
    /// 继承该类以创建可序列化的类.    
    /// </summary>
    [Serializable]
    public abstract class MobileObject : IMobileObject
    {
        #region Serialize

        void IMobileObject.GetChildren(SerializationInfo info, MobileFormatter formatter)
        {
            OnGetChildren(info, formatter);
        }

        void IMobileObject.GetState(SerializationInfo info)
        {
            OnGetState(info, StateMode.Serialization);
        }

        /// <summary>
        /// Override this method to insert your field values
        /// into the MobileFormatter serialzation stream.
        /// </summary>
        /// <param name="info">
        /// Object containing the data to serialize.
        /// </param>
        /// <param name="mode">
        /// The StateMode indicating why this method was invoked.
        /// </param>
        [EditorBrowsable(EditorBrowsableState.Advanced)]
        protected virtual void OnGetState(SerializationInfo info, StateMode mode)
        { }

        /// <summary>
        /// Override this method to insert your child object
        /// references into the MobileFormatter serialzation stream.
        /// </summary>
        /// <param name="info">
        /// Object containing the data to serialize.
        /// </param>
        /// <param name="formatter">
        /// Reference to MobileFormatter instance. Use this to
        /// convert child references to/from reference id values.
        /// </param>
        [EditorBrowsable(EditorBrowsableState.Advanced)]
        protected virtual void OnGetChildren(SerializationInfo info, MobileFormatter formatter)
        { }

        #endregion

        #region Deserialize

        void IMobileObject.SetState(SerializationInfo info)
        {
            OnSetState(info, StateMode.Serialization);
        }

        void IMobileObject.SetChildren(SerializationInfo info, MobileFormatter formatter)
        {
            OnSetChildren(info, formatter);
        }

        /// <summary>
        /// Override this method to retrieve your field values
        /// from the MobileFormatter serialzation stream.
        /// </summary>
        /// <param name="info">
        /// Object containing the data to serialize.
        /// </param>
        /// <param name="mode">
        /// The StateMode indicating why this method was invoked.
        /// </param>
        [EditorBrowsable(EditorBrowsableState.Advanced)]
        protected virtual void OnSetState(SerializationInfo info, StateMode mode)
        { }

        /// <summary>
        /// Override this method to retrieve your child object
        /// references from the MobileFormatter serialzation stream.
        /// </summary>
        /// <param name="info">
        /// Object containing the data to serialize.
        /// </param>
        /// <param name="formatter">
        /// Reference to MobileFormatter instance. Use this to
        /// convert child references to/from reference id values.
        /// </param>
        [EditorBrowsable(EditorBrowsableState.Advanced)]
        protected virtual void OnSetChildren(SerializationInfo info, MobileFormatter formatter)
        { }

        #endregion
    }
}
