﻿using System;
using System.Collections.Generic;
using Wcdf.Serialization.Mobile;
using Wcdf.Serialization;
using System.ComponentModel;
using Wcdf.Properties;

namespace Wcdf.Core
{
    /// <summary>
    /// 继承本类容易的创建可序列化列表类.    
    /// </summary>
    /// <typeparam name="T">
    /// Type of the items contained in the list.
    /// </typeparam>
#if TESTING
  [System.Diagnostics.DebuggerStepThrough]
#endif
    [Serializable]
    public class MobileObservableCollection<T> : System.Collections.ObjectModel.ObservableCollection<T>,
      IMobileObject
    {
        #region IMobileObject Members

        void IMobileObject.GetChildren(SerializationInfo info, MobileFormatter formatter)
        {
            OnGetChildren(info, formatter);
        }

        void IMobileObject.GetState(SerializationInfo info)
        {
            OnGetState(info);
        }

        /// <summary>
        /// Override this method to get custom field values
        /// from the serialization stream.
        /// </summary>
        /// <param name="info">Serialization info.</param>
        [EditorBrowsable(EditorBrowsableState.Advanced)]
        protected virtual void OnGetState(SerializationInfo info)
        { }

        /// <summary>
        /// Override this method to get custom child object
        /// values from the serialization stream.
        /// </summary>
        /// <param name="info">Serialization info.</param>
        /// <param name="formatter">Reference to the MobileFormatter.</param>
        [EditorBrowsable(EditorBrowsableState.Advanced)]
        protected virtual void OnGetChildren(SerializationInfo info, MobileFormatter formatter)
        {
            if (!typeof(IMobileObject).IsAssignableFrom(typeof(T)))
                throw new InvalidOperationException(Resources.CannotSerializeCollectionsNotOfIMobileObject);

            List<int> references = new List<int>();
            for (int x = 0; x < this.Count; x++)
            {
                T child = this[x];
                if (child != null)
                {
                    SerializationInfo childInfo = formatter.SerializeObject(child);
                    references.Add(childInfo.ReferenceId);
                }
            }
            if (references.Count > 0)
                info.AddValue("$list", references);
        }

        void IMobileObject.SetState(SerializationInfo info)
        {
            OnSetState(info);
        }

        void IMobileObject.SetChildren(SerializationInfo info, MobileFormatter formatter)
        {
            OnSetChildren(info, formatter);
        }

        /// <summary>
        /// Override this method to set custom field values
        /// into the serialization stream.
        /// </summary>
        /// <param name="info">Serialization info.</param>
        [EditorBrowsable(EditorBrowsableState.Advanced)]
        protected virtual void OnSetState(SerializationInfo info)
        { }

        /// <summary>
        /// Override this method to set custom child object
        /// values into the serialization stream.
        /// </summary>
        /// <param name="info">Serialization info.</param>
        /// <param name="formatter">Reference to the MobileFormatter.</param>
        [EditorBrowsable(EditorBrowsableState.Advanced)]
        protected virtual void OnSetChildren(SerializationInfo info, MobileFormatter formatter)
        {
            if (!typeof(IMobileObject).IsAssignableFrom(typeof(T)))
                throw new InvalidOperationException(Resources.CannotSerializeCollectionsNotOfIMobileObject);

            if (info.Values.ContainsKey("$list"))
            {
                List<int> references = (List<int>)info.Values["$list"].Value;
                foreach (int reference in references)
                {
                    T child = (T)formatter.GetObject(reference);
                    this.Add(child);
                }
            }
        }

        #endregion
    }
}
