﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Wcdf.Properties;
using Wcdf.Serialization.Mobile;
using Wcdf.Serialization;

namespace Wcdf.Core
{
    /// <summary>
    /// 基于ObservableBindingList的一种只读类
    /// </summary>
    /// <typeparam name="C">列表项.</typeparam>
    /// <remarks>
    /// 通过IsReadOnly可解锁列表的增加,删除能力.    
    /// </remarks>
    [Serializable]
    public class ReadOnlyObservableBindingList<C> : ObservableBindingList<C>,
      Core.IReadOnlyBindingList
    {
        /// <summary>
        /// Creates an instance of the object.
        /// </summary>
        public ReadOnlyObservableBindingList()
        {
            AllowEdit = false;
            AllowNew = false;
            AllowRemove = false;

            this.CollectionChanged += (o, e) =>
            {
                if (IsReadOnly)
                    throw new NotSupportedException(Resources.ChangeReadOnlyListInvalid);
            };
        }

        #region IsReadOnly

        private bool _isReadOnly = true;

        /// <summary>
        /// Gets or sets a value indicating whether the list is readonly.
        /// </summary>
        /// <remarks>
        /// Subclasses can set this value to unlock the collection
        /// in order to alter the collection's data.
        /// </remarks>
        /// <value>True indicates that the list is readonly.</value>
        public bool IsReadOnly
        {
            get { return _isReadOnly; }
            protected set { _isReadOnly = value; }
        }

        bool Core.IReadOnlyBindingList.IsReadOnly
        {
            get { return IsReadOnly; }
            set { IsReadOnly = value; }
        }

        #endregion

        #region MobileFormatter

        /// <summary>
        /// Override this method to insert your field values
        /// into the MobileFormatter serialzation stream.
        /// </summary>
        /// <param name="info">
        /// Object containing the data to serialize.
        /// </param>
        protected override void OnGetState(SerializationInfo info)
        {
            base.OnGetState(info);
            info.AddValue("Csla.Core.ReadOnlyBindingList._isReadOnly", _isReadOnly);
        }

        /// <summary>
        /// Override this method to retrieve your field values
        /// from the MobileFormatter serialzation stream.
        /// </summary>
        /// <param name="info">
        /// Object containing the data to serialize.
        /// </param>
        protected override void OnSetState(SerializationInfo info)
        {
            base.OnSetState(info);
            _isReadOnly = info.GetValue<bool>("Csla.Core.ReadOnlyBindingList._isReadOnly");
        }

        /// <summary>
        /// Override this method to retrieve your child object
        /// references from the MobileFormatter serialzation stream.
        /// </summary>
        /// <param name="info">
        /// Object containing the data to serialize.
        /// </param>
        /// <param name="formatter">
        /// Reference to MobileFormatter instance. Use this to
        /// convert child references to/from reference id values.
        /// </param>
        protected override void OnSetChildren(SerializationInfo info, MobileFormatter formatter)
        {
            var old = IsReadOnly;
            IsReadOnly = false;
            base.OnSetChildren(info, formatter);
            IsReadOnly = old;
        }

        #endregion
    }
}
