﻿using System;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Reflection;

namespace Wcdf.Data
{
    internal class ConditionBuilder : ExpressionVisitor
    {
        //private List<object> m_arguments;
        private Stack<string> _conditionParts;

        public string Condition { get; private set; }

        //public object[] Arguments { get; private set; }

        public void Build(Expression expression)
        {
            PartialEvaluator evaluator = new PartialEvaluator();
            Expression evaluatedExpression = evaluator.Eval(expression);

            //this.m_arguments = new List<object>();
            this._conditionParts = new Stack<string>();

            this.Visit(evaluatedExpression);

            //this.Arguments = this.m_arguments.ToArray();
            this.Condition = this._conditionParts.Count > 0 ? this._conditionParts.Pop() : null;
        }

        /// <summary>
        /// 重写二元表达式
        /// </summary>
        /// <param name="b"></param>
        /// <returns></returns>
        protected override Expression VisitBinary(BinaryExpression b)
        {
            if (b == null) return b;

            string opr;
            switch (b.NodeType)
            {
                case ExpressionType.Equal:
                    opr = "=";
                    break;
                case ExpressionType.NotEqual:
                    opr = "<>";
                    break;
                case ExpressionType.GreaterThan:
                    opr = ">";
                    break;
                case ExpressionType.GreaterThanOrEqual:
                    opr = ">=";
                    break;
                case ExpressionType.LessThan:
                    opr = "<";
                    break;
                case ExpressionType.LessThanOrEqual:
                    opr = "<=";
                    break;
                case ExpressionType.AndAlso:
                    opr = "AND";
                    break;
                case ExpressionType.OrElse:
                    opr = "OR";
                    break;
                case ExpressionType.Add:
                    opr = "+";
                    break;
                case ExpressionType.Subtract:
                    opr = "-";
                    break;
                case ExpressionType.Multiply:
                    opr = "*";
                    break;
                case ExpressionType.Divide:
                    opr = "/";
                    break;
                default:
                    throw new NotSupportedException(string.Format(Wcdf.Properties.Resources.LinqOpNoSupported, b.NodeType));
            }

            this.Visit(b.Left);
            this.Visit(b.Right);

            string right = this._conditionParts.Pop();
            string left = this._conditionParts.Pop();

            string condition = String.Format("({0} {1} {2})", left, opr, right);
            this._conditionParts.Push(condition);

            return b;
        }

        /// <summary>
        /// 常量
        /// </summary>
        /// <param name="c"></param>
        /// <returns></returns>
        protected override Expression VisitConstant(ConstantExpression c)
        {
            if (c == null) return c;

            //this.m_arguments.Add(c.Value);
            //this._conditionParts.Push(String.Format("{{{0}}}", this.m_arguments.Count - 1));
            this._conditionParts.Push(TypeResolver.ResolveToSqlString(c.Value));

            return c;
        }

        /// <summary>
        /// 成员(字段名)
        /// </summary>
        /// <param name="m"></param>
        /// <returns></returns>
        protected override Expression VisitMemberAccess(MemberExpression m)
        {
            if (m == null) return m;

            PropertyInfo propertyInfo = m.Member as PropertyInfo;
            if (propertyInfo == null) return m;

            this._conditionParts.Push(String.Format("[{0}]", propertyInfo.Name));

            return m;
        }

        protected override Expression VisitMethodCall(MethodCallExpression m)
        {
            string opr;
            string strF = "({0} {1} {2})";
            this.Visit(m.Object);
            this.Visit(m.Arguments[0]);
            string right = m.Object != null ? this._conditionParts.Pop() : null;
            string left = this._conditionParts.Pop();

            switch (m.Method.Name)
            {
                case "Contains":
                    opr = "like";
                    right = "'%'+" + right + "+'%'";
                    break;
                case "StartsWith":
                    opr = "like";
                    right = right + "+'%'";
                    break;
                case "EndWith":
                    opr = "like";
                    right = "'%'+" + right;
                    break;
                case "IsNullOrEmpty":
                    opr = "or";
                    strF = "({0} is null or {0}='')";
                    break;
                case "Equals":
                    opr = "=";
                    strF = "{0} {1} {2}";
                    break;
                default:
                    throw new NotSupportedException(string.Format(Wcdf.Properties.Resources.LinqOpNoSupported, m.Method.Name));
            }

            string condition = String.Format(strF, left, opr, right);
            this._conditionParts.Push(condition);

            return m;
            //return base.VisitMethodCall(m);
        }

        protected override Expression VisitUnary(UnaryExpression u)
        {
            string opr;
            switch (u.NodeType)
            {
                case ExpressionType.Not:
                    opr = "Not";
                    break;
                default:
                    throw new NotSupportedException(string.Format(Wcdf.Properties.Resources.LinqOpNoSupported, u.NodeType));
            }
            this.Visit(u.Operand);
            string right = this._conditionParts.Pop();
            string condition = string.Format("{0}({1})", opr, right);
            this._conditionParts.Push(condition);
            return u;
        }
    }
}
