﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Wcdf.Data.SqlCommandBuilder
{
    /// <summary>
    /// 删除语句命令构造类
    /// </summary>
    public class DeleteCommand : ISqlCommand
    {
        //数据表名
        private string _DTName;
        private string _sqlText;

        public DeleteCommand(string datatableName, Criteria cri)
        {
            _DTName = datatableName;
            ApplyDataRight(_DTName);
            _sqlText = BuildSql(datatableName, cri.Filters.Condition);
        }
        #region
        public string PrivoderName { get; set; }

        public string CommandText
        {
            get { return this._sqlText; }
        }

        public void BuildCommand(System.Data.Common.DbCommand comm)
        {
            comm.CommandText = this.CommandText;
            comm.CommandType = System.Data.CommandType.Text;
        }
        #endregion

        private string BuildSql(string tablename, string condition)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("delete ");
            sb.Append(tablename);
            sb.Append(" where ");
            sb.Append(condition);
            return sb.ToString();
        }

        /// <summary>
        /// 权限判断
        /// </summary>
        private void ApplyDataRight(string objectName)
        {
            Wcdf.Rules.ObjectRight or = Wcdf.Rules.DataPermissionManager.Instance.GetOrAddObjectRight(ApplicationContext.User.Identity.Name, objectName, Rules.AuthorizationActions.DeleteObject);
            if (or != null && or.IsControl)
            {
                if (!or.IsCanDelete)
                {
                    throw new Exception(string.Format(Wcdf.Properties.Resources.CanNotDeleteAuthorization, objectName));
                }
            }
        }
    }
}
