﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Wcdf.Core.FieldManager;
using Wcdf.Rules;
using System.Configuration;
using Wcdf.Properties;
using Wcdf.Core;
using System.Data.Common;

namespace Wcdf.Data.SqlCommandBuilder
{
    /// <summary>
    /// 
    /// </summary>
    internal class InsertCommand : ISqlCommand
    {
        private string _tableName;
        private string _sqlText;
        private Dictionary<IPropertyInfo, object> _changeData;
        public InsertCommand(string tableName, PropertyInfoList propertys, FieldDataManager fieldData)
        {
            _tableName = tableName;
            ApplyDataRight(tableName, propertys, fieldData);
        }

        #region ISqlCommand
        private string _privoderName;
        public string PrivoderName
        {
            get
            {
                if (_privoderName == null)
                {
                    _privoderName = ConfigurationManager.ConnectionStrings[DataAccess.DefaultDataBase].ProviderName;
                }
                return _privoderName;
            }
            set
            {
                _privoderName = value;
            }
        }

        /// <summary>
        /// 该属性需要在BuildCommand()方法后才会有相应COMM的SQL语句.
        /// 但语句并不是最终发向数据库的命令.
        /// </summary>
        public string CommandText
        {
            get { return this._sqlText; }
        }

        public void BuildCommand(System.Data.Common.DbCommand comm)
        {
            _sqlText = BuildSql(comm);
            comm.CommandType = System.Data.CommandType.Text;
            comm.CommandText = _sqlText;
        }
        #endregion

        private Dictionary<IPropertyInfo, object> GetChange(List<IPropertyInfo> propertys, FieldDataManager fieldData)
        {
            Dictionary<IPropertyInfo, object> result = new Dictionary<IPropertyInfo, object>();
            //var dirtyData = fieldData.GetFeildDatas();
            foreach (var colinfo in propertys)
            {
                var data = fieldData.GetFieldData(colinfo);
                if (data != null && data.IsDirty)
                {
                    result.Add(colinfo, data.Value);
                }
            }
            return result;
        }

        private void ApplyDataRight(string objectName, List<Core.IPropertyInfo> colinfoList, FieldDataManager fdm)
        {
            var col = colinfoList.FindAll(o => !string.IsNullOrEmpty(o.FriendlyName));
            _changeData = this.GetChange(col, fdm);

            ObjectRight or = Wcdf.Rules.DataPermissionManager.Instance.GetOrAddObjectRight(ApplicationContext.User.Identity.Name, objectName, Rules.AuthorizationActions.CreateObject);
            if (or != null && or.IsControl)
            {
                if (or != null && or.IsControl)
                {
                    if (!or.IsCanInsert)
                    {
                        throw new Exception(string.Format(Resources.CanNotInsertAuthorization, objectName));
                    }

                    foreach (FieldRight fr in or.FieldRightList)
                    {
                        //不可为空
                        if (fr.IsUnnullable)
                        {
                            var propertyinfo = _changeData.Keys.FirstOrDefault(o => o.FriendlyName == fr.FieldName);
                            var value = _changeData[propertyinfo];

                            if ((value == null) || string.Empty.Equals(value) || Convert.IsDBNull(value))
                                throw new System.Exception(string.Format(Resources.CanNotSetNullOrEmpty, fr.FieldName));
                        }
                    }
                }
            }

        }

        private string BuildSql(System.Data.Common.DbCommand comm)
        {
            StringBuilder cols = new StringBuilder();
            StringBuilder sqlPara = new StringBuilder();
            foreach (var col in _changeData)
            {
                cols.Append(",");
                cols.Append(col.Key.FriendlyName);
                string paraName = DataMapper.ParameterCreator(this.PrivoderName, col.Key.FriendlyName);
                sqlPara.Append(",");
                sqlPara.Append(paraName);

                DbParameter dbPara = comm.CreateParameter();
                dbPara.ParameterName = paraName;
                dbPara.Value = col.Value;
                comm.Parameters.Add(dbPara);

            }
            //去掉首个","
            cols.Remove(0, 1);
            sqlPara.Remove(0, 1);
            StringBuilder sb = new StringBuilder();
            sb.Append("insert ");
            sb.Append(_tableName);
            sb.Append("(");
            sb.Append(cols.ToString());
            sb.Append(")");
            sb.Append("values(");
            sb.Append(sqlPara.ToString());
            sb.Append(")");
            return sb.ToString();
        }
    }
}
