﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Wcdf.Core.FieldManager;
using Wcdf.Rules;

namespace Wcdf.Data.SqlCommandBuilder
{
    /// <summary>
    /// 查询语句构造类
    /// </summary>
    internal class QueryCommand : ISqlCommand
    {
        string _tableName;
        string _where;
        string _orderby;
        string _cols;
        Paging _paging;
        string _fistField;
        string _sqlText;
        /// <summary>
        /// 构造器
        /// </summary>
        /// <param name="cols"></param>
        /// <param name="mapping"></param>
        /// <param name="filter"></param>
        /// <param name="orderby"></param>
        public QueryCommand(string dataTableName, string[] cols, PropertyInfoList mapping, Criteria cri)
        {
            ApplyDataRight(mapping, cols, cri.Filters);
            //_where = null == filter ? string.Empty : Filter.FilterAdapter(filter, mapping);
            _orderby = null == cri.OrderBy ? string.Empty : string.Join(",", cri.OrderBy.OrderByString);
            _tableName = dataTableName;
            _paging = cri.Paging;
            if (null != _paging)
                _sqlText = BuildPagingSql();
            else
                _sqlText = BuildSql();
        }

        /// <summary>
        /// 分页查询
        /// </summary>
        /// <returns></returns>
        public string BuildPagingSql()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("WITH pageobject AS ");
            sb.AppendFormat("(SELECT {0},", _cols);
            sb.AppendFormat("        ROW_NUMBER() OVER (ORDER BY {0}) as RowNumber ", string.IsNullOrEmpty(_orderby) ? _fistField : _orderby);
            sb.AppendFormat("   FROM {0}(nolock)", _tableName);
            sb.AppendFormat("  WHERE {0} )", string.IsNullOrEmpty(_where) ? " 1=1 " : _where);
            sb.AppendFormat("SELECT TOP {0} {1} FROM pageobject", _paging.PageSize, _cols);
            sb.AppendFormat(" WHERE RowNumber >{0}", _paging.PageSize * (_paging.PageIndex - 1));
            sb.AppendFormat(" ORDER BY {0}", string.IsNullOrEmpty(_orderby) ? _fistField : _orderby);
            sb.Append(";");
            sb.Append("SELECT COUNT(*) AS [value]");
            sb.Append("  FROM " + _tableName + "(nolock) ");
            sb.AppendFormat(" WHERE {0}", string.IsNullOrEmpty(_where) ? "1=1" : _where);
            return sb.ToString();
        }

        private void ApplyDataRight(PropertyInfoList mapping, string[] cols, Filter filter)
        {
            if (cols == null || cols.Length == 0)
                cols = new string[] { "*" };
            List<string> colList = cols.ToList();
            //if exist *;
            int sindex = colList.IndexOf("*");
            if (sindex != -1)
            {
                colList.RemoveAt(sindex);
                var query = from m in mapping
                            where !string.IsNullOrEmpty(m.FriendlyName)
                            select m.FriendlyName;
                colList.AddRange(query);
            }
            ObjectRight or = DataPermissionManager.Instance.GetOrAddObjectRight(ApplicationContext.User.Identity.Name, _tableName, AuthorizationActions.ReadProperty);
            string rightFilter = string.Empty;
            if (null != or && or.IsControl)
            {
                if (!or.IsCanSelect)
                    throw new Exception(Wcdf.Properties.Resources.CanNotReadAuthorization);
                foreach (FieldRight field in or.FieldRightList)
                {
                    if (field.IsUnvisible)
                    {
                        colList.Remove(field.FieldName);
                        colList.Add(string.Format("{0}=Null", field.FieldName));
                    }
                    if (!string.IsNullOrEmpty(field.SelectCondition))
                    {
                        Filter drf = new Filter(field.SelectCondition);
                        filter.ChildFilter.Add(drf);
                    }
                }
            }
            _where = null == filter ? string.Empty : filter.GetCondition();
            _cols = string.Join(",", colList.ToArray());
            _fistField = colList[0];
        }

        /// <summary>
        /// 构建查询语句
        /// </summary>
        /// <returns></returns>
        public string BuildSql()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("SELECT ");
            sb.Append(_cols);
            sb.Append("  FROM [" + _tableName + "]");
            sb.Append(string.IsNullOrEmpty(_where) ? "" : " WHERE ");
            sb.Append(_where);
            sb.Append(string.IsNullOrEmpty(_orderby) ? "" : " ORDER BY ");
            sb.Append(_orderby);
            return sb.ToString();
        }

        #region ISqlCommand 成员
        public string PrivoderName { get; set; }

        public string CommandText
        {
            get { return _sqlText; }
        }        
        /// <summary>
        /// 创建查询命令
        /// </summary>
        /// <param name="comm"></param>
        public void BuildCommand(System.Data.Common.DbCommand comm)
        {
            comm.CommandText = this.CommandText;            
            comm.CommandType = System.Data.CommandType.Text;
        }

        #endregion
    }
}
