﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;
using Wcdf.Core.FieldManager;
using Wcdf.Rules;
using Wcdf.Properties;
using Wcdf.Core;
using System.Data.Common;

namespace Wcdf.Data.SqlCommandBuilder
{
    /// <summary>
    /// 更新命令
    /// </summary>
    internal class UpdateCommand : ISqlCommand
    {
        private string _tableName;
        private string _sqlText;
        private Dictionary<IPropertyInfo, object> _changeData;
        private Filter _primaryKeyFilter;

        public UpdateCommand(string tableName, PropertyInfoList propertys, FieldDataManager fieldData)
        {
            _tableName = tableName;
            _primaryKeyFilter = new Filter();
            ApplyDataRight(tableName, propertys, fieldData);
        }

        #region ISqlCommand
        private string _privoderName;
        public string PrivoderName
        {
            get
            {
                if (_privoderName == null)
                {
                    _privoderName = ConfigurationManager.ConnectionStrings[DataAccess.DefaultDataBase].ProviderName;
                }
                return _privoderName;
            }
            set
            {
                _privoderName = value;
            }
        }

        /// <summary>
        /// 该属性需要在BuildCommand()方法后才会有相应COMM的SQL语句.
        /// 但语句并不是最终发向数据库的命令.
        /// </summary>
        public string CommandText
        {
            get { return _sqlText; }
        }

        public void BuildCommand(System.Data.Common.DbCommand comm)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("update ");
            sb.Append(_tableName);
            sb.Append(" set ");
            foreach (var col in _changeData)
            {
                string parameterName = DataMapper.ParameterCreator(this.PrivoderName, col.Key.FriendlyName);
                sb.Append(col.Key.FriendlyName + " = ");
                sb.Append(parameterName);
                sb.Append(",");

                DbParameter dbPara = comm.CreateParameter();
                dbPara.ParameterName = parameterName;
                dbPara.Value = col.Value;
                comm.Parameters.Add(dbPara);
            }
            //去掉最后一个","
            sb.Remove(sb.Length - 1, 1);
            sb.Append(" where ");
            sb.Append(_primaryKeyFilter.GetCondition());

            _sqlText = sb.ToString();
            comm.CommandType = System.Data.CommandType.Text;
            comm.CommandText = _sqlText;
        }
        #endregion

        private Dictionary<IPropertyInfo, object> GetChange(List<IPropertyInfo> propertys, FieldDataManager fieldData)
        {
            Dictionary<IPropertyInfo, object> result = new Dictionary<IPropertyInfo, object>();
            //var dirtyData = fieldData.GetFeildDatas();
            foreach (var colinfo in propertys)
            {
                if (colinfo.RelationshipType == RelationshipTypes.PrimaryKey)
                {
                    var data = fieldData.GetFieldData(colinfo);
                    string strfilter = string.Format("{0}={1}", colinfo.FriendlyName, TypeResolver.ResolveToSqlString(data.Value));
                    _primaryKeyFilter.ChildFilter.Add(new Filter(strfilter));
                }
                else
                {
                    var data = fieldData.GetFieldData(colinfo);
                    if (data != null && data.IsDirty)
                    {
                        result.Add(colinfo, data.Value);
                    }
                }
            }
            return result;
        }

        private void ApplyDataRight(string objectName, PropertyInfoList propertys, FieldDataManager fdm)
        {
            var col = propertys.FindAll(o => !string.IsNullOrEmpty(o.FriendlyName));
            _changeData = this.GetChange(col, fdm);

            ObjectRight or = Wcdf.Rules.DataPermissionManager.Instance.GetOrAddObjectRight(ApplicationContext.User.Identity.Name, objectName, Rules.AuthorizationActions.EditObject);
            if (or != null && or.IsControl)
            {
                if (or != null && or.IsControl)
                {
                    if (!or.IsCanUpdate)
                    {
                        throw new Exception(string.Format(Resources.CanNotEditAuthorization, objectName));
                    }

                    foreach (FieldRight fr in or.FieldRightList)
                    {
                        //只读
                        if (fr.IsReadOnly || fr.IsUnnullable)
                        {
                            var propertyinfo = _changeData.Keys.FirstOrDefault(o => o.FriendlyName == fr.FieldName);
                            //如果为只读.propertyinfo应该不存在
                            if (fr.IsReadOnly && propertyinfo != null)
                                throw new Exception(string.Format(Wcdf.Properties.Resources.CanNotEditAuthorization, fr.FieldName));

                            var value = _changeData[propertyinfo];
                            if (fr.IsUnnullable &&
                                ((value == null) || string.Empty.Equals(value) || Convert.IsDBNull(value)))
                                throw new System.Exception(string.Format(Resources.CanNotSetNullOrEmpty, fr.FieldName));
                        }
                    }
                }
            }
        }

    }
}
