﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

namespace Wcdf.Data
{
    /// <summary>
    /// 数据类型注册类,将系统的数据类型与数据库相关联
    /// </summary>
    public class TypeResolver
    {
        private static IDictionary _types = new Hashtable();

        private TypeResolver() { }
        /// <summary>
        /// 初始化类型集
        /// </summary>
        static TypeResolver()
        {
            _types.Add(typeof(Guid), DBType.Guid);
            _types.Add(typeof(string), DBType.String);
            _types.Add(typeof(int), DBType.Int);
            _types.Add(typeof(int?), DBType.Int);
            _types.Add(typeof(uint), DBType.Int);
            _types.Add(typeof(uint?), DBType.Int);
            _types.Add(typeof(long), DBType.BigInt);
            _types.Add(typeof(long?), DBType.BigInt);
            _types.Add(typeof(ulong), DBType.BigInt);
            _types.Add(typeof(ulong?), DBType.BigInt);
            _types.Add(typeof(decimal), DBType.Decimal);
            _types.Add(typeof(decimal?), DBType.Decimal);
            _types.Add(typeof(float), DBType.Float);
            _types.Add(typeof(float?), DBType.Float);
            _types.Add(typeof(double), DBType.Double);
            _types.Add(typeof(double?), DBType.Double);
            _types.Add(typeof(char), DBType.String);
            _types.Add(typeof(char?), DBType.String);
            _types.Add(typeof(byte), DBType.Int);
            _types.Add(typeof(byte?), DBType.Int);
            _types.Add(typeof(bool), DBType.Boolean);
            _types.Add(typeof(bool?), DBType.Boolean);
            _types.Add(typeof(byte[]), DBType.Blob);
            _types.Add(typeof(byte?[]), DBType.Blob);
            _types.Add(typeof(DateTime), DBType.DateTime);
            _types.Add(typeof(DateTime?), DBType.DateTime);
        }

        /// <summary>
        /// 将系统数据类型转换为DBType类型
        /// </summary>
        /// <param name="type">系统数据类型</param>
        /// <returns></returns>
        public static DBType ResolveType(Type type)
        {
            if (_types.Contains(type))
                return (DBType)_types[type];
            else return DBType.Variant;
        }

        /// <summary>
        /// 将值转换为对应的SQL的字符串形式.
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        public static string ResolveToSqlString(object value)
        {
            switch (ResolveType(value.GetType()))
            {
                case DBType.DateTime:
                    //时间类型由于比较灵活,需要处理时间格式
                    if (!(value is DateTime))
                        //DateTime dt = (DateTime)value;                    
                        new NotSupportedException(Wcdf.Properties.Resources.CannotResolvedSQL);
                    return string.Format("'{0}'", ((DateTime)value).ToString("yyyy.MM.dd HH:mm:ss"));
                case DBType.BigInt:
                case DBType.Decimal:
                case DBType.Double:
                case DBType.Float:
                case DBType.Int:
                    return value.ToString();
                case DBType.String:
                case DBType.Text:
                case DBType.Guid:
                    return string.Format("'{0}'", value.ToString());
                default:
                    throw new NotSupportedException(Wcdf.Properties.Resources.CannotResolvedSQL);
            }
        }
    }
}
