﻿using Wcdf.Server;
using System.Collections.Generic;
using System.ServiceModel;
using System;
using Wcdf.DataPortalClient;
using Wcdf.Server.Hosts.WcfChannel;
using Wcdf.Server.Hosts;
using Wcdf.Serialization.Mobile;
using Wcdf.Core;
using Wcdf.Data;
using System.ServiceModel.Description;
using Wcdf.Security;

namespace Wcdf.DataPortalClient
{
    /// <summary>
    /// Wcf门户代理,具有约定的客户端与服务端的处理方式   
    /// </summary>
    /// <remarks>
    /// UI项目命名方式:项目名称+.Client,例EmailManager.Client
    /// 中间接口命名方式:项目名称+.Common,例EmailManager.Common
    /// 服务端命名方式:项目名称+.Service,例EmailManager.Service
    /// IIS服务宿主文件的命名方式:以中间接口项目的命名空间命名,例邮件服务文件为:EmailManager.Common.svc
    /// </remarks>
    public class WcfProxy : IDataPortalProxy
    {
        private static Dictionary<string, ChannelFactory> channelFactories = new Dictionary<string, ChannelFactory>();
        private static object syncHelper = new object();

        private const string _endPoint = "default";
        public const string DefaultHttpBinding = "defaultHttpBinding";
        /// <summary>
        /// 
        /// 通过配置节点名,从通道工厂缓存中取得对应的Channel
        /// </summary>
        /// <typeparam name="TChannel"></typeparam>
        /// <param name="moduleName">配置节点名</param>
        /// <returns></returns>
        private static ChannelFactory<TChannel> GetChannelFactory<TChannel>(string moduleName)
        {
            string portalUrl, key, endpointConfigurationName = string.Empty;
            Type tchType = typeof(TChannel);
            string strType = tchType.ToString();

            if (string.IsNullOrEmpty(moduleName))
            {
                moduleName = tchType.Module.Name;
            }
            if (Wcdf.DataPortalClient.Wcf.ClientConfig.Config.ContainsKey(moduleName))
            {
                //取得基地址
                var moduleinfo = Wcdf.DataPortalClient.Wcf.ClientConfig.Config[moduleName];
                portalUrl = moduleinfo.DataPortalUrlString;

                if (channelFactories.ContainsKey(portalUrl + strType))
                    return channelFactories[portalUrl] as ChannelFactory<TChannel>;
                //是否有服务列表
                if (moduleinfo.ServiceInfoList.ContainsKey(strType))
                {
                    endpointConfigurationName = moduleinfo.ServiceInfoList[strType].Endpoint;
                }
            }
            else
            {
                portalUrl = ApplicationContext.DataPortalUrlString;
            }

            key = portalUrl + strType;

            ChannelFactory<TChannel> channelFactory = null;
            if (channelFactories.ContainsKey(key))
            {
                channelFactory = channelFactories[key] as ChannelFactory<TChannel>;
            }
            if (channelFactory == null)
            {
                string serviceUrl = portalUrl + tchType.FullName + ".svc";
                EndpointAddress url = new EndpointAddress(serviceUrl);
                if (string.IsNullOrEmpty(endpointConfigurationName))
                {
                    //如果都没有，就默认为配置在
                    //channelFactory = new ChannelFactory<TChannel>(GetDefautEndpoint(serviceUrl, tchType));
                    endpointConfigurationName = tchType.FullName;
                }
                channelFactory = new ChannelFactory<TChannel>(endpointConfigurationName, url);
                //channelFactory.Endpoint.
                ////附加终结点信息...以下改用配置文件实现.
                //foreach (OperationDescription opesc in channelFactory.Endpoint.Contract.Operations)
                //{
                //    FaultDescription faultDescription = new FaultDescription(ServiceExceptionDetail.FaultAction);
                //    faultDescription.Name = "ServiceExceptionDetail";
                //    faultDescription.ProtectionLevel = System.Net.Security.ProtectionLevel.None;
                //    faultDescription.DetailType = typeof(ServiceExceptionDetail);
                //    opesc.Faults.Add(faultDescription);
                //}
                lock (syncHelper)
                {
                    channelFactories[key] = channelFactory;
                }

            }
            return channelFactory;
        }

        /// <summary>
        /// 终结点配置信息
        /// </summary>
        /// <param name="TChannel"></param>
        /// <returns></returns>
        private static ServiceEndpoint GetDefautEndpoint(string serviceUrl, Type TChannel)
        {
            EndpointAddress endPoint = new EndpointAddress(serviceUrl);
            System.ServiceModel.Channels.Binding bind = new WSHttpBinding(DefaultHttpBinding);
            ContractDescription contract = ContractDescription.GetContract(TChannel);
            foreach (OperationDescription opesc in contract.Operations)
            {
                FaultDescription faultDescription = new FaultDescription(ServiceExceptionDetail.FaultAction);
                faultDescription.DetailType = typeof(ServiceExceptionDetail);
                opesc.Faults.Add(faultDescription);
            }
            ServiceEndpoint endpoint = new ServiceEndpoint(contract, bind, endPoint);
            endpoint.Behaviors.Add(new Wcdf.DataPortalClient.Wcf.ClientEndpointBehavior());
            return endpoint;
        }


        #region IDataPortalProxy Members

        public bool IsServerRemote
        {
            get { return true; }
        }
        /// <summary>
        /// 调用方法
        /// </summary>
        /// <typeparam name="TChannel">WCF形式声明的接口类型</typeparam>
        /// <param name="action">接口方法委托</param>
        /// <param name="moduleName">配置模块名</param>
        public void Invoke<TChannel>(System.Action<TChannel> action, string moduleName)
        {
            ChannelFactory<TChannel> factory = GetChannelFactory<TChannel>(moduleName);
            ICommunicationObject channel = null;
            try
            {
                TChannel proxy = factory.CreateChannel();
                channel = proxy as ICommunicationObject;
                //using (OperationContextScope contextScope = new OperationContextScope(proxy as IContextChannel))
                //{
                //    var msg = new DataPortalContext(GetPrincipal(), true);
                //    MessageHeader<DataPortalContext> header = new MessageHeader<DataPortalContext>(msg);
                //    OperationContext.Current.OutgoingMessageHeaders.Add(header.GetUntypedHeader(DataPortalContext.Name, DataPortalContext.NameSpace));
                    
                //}
                action(proxy);
                channel.Close();
            }
            catch (CommunicationException ex)
            {
                if (null != channel)
                {
                    channel.Abort();
                }
                factory = null;
                throw ex;
            }
            catch (TimeoutException ex)
            {
                if (null != channel)
                    channel.Abort();
                throw ex;
            }
            catch (System.Exception ex)
            {
                if (null != channel)
                    channel.Close();
                throw ex;
            }
        }
        /// <summary>
        /// 调用方法
        /// </summary>
        /// <typeparam name="TChannel">WCF形式声明的接口类型</typeparam>
        /// <typeparam name="TReturn">返回类型</typeparam>
        /// <param name="func">接口方法委托</param>
        /// <param name="moduleName">配置模块名</param>
        /// <returns>返回TReturn类型</returns>
        public TReturn Invoke<TChannel, TReturn>(System.Func<TChannel, TReturn> func, string moduleName)
        {
            TReturn result = default(TReturn);
            ChannelFactory<TChannel> factory = GetChannelFactory<TChannel>(moduleName);
            ICommunicationObject channel = null;
            try
            {               
                TChannel proxy = factory.CreateChannel();
                channel = proxy as ICommunicationObject;
                result = func(proxy);
                channel.Close();
            }
            catch (CommunicationException ex)
            {
                if (null != channel)
                {
                    channel.Abort();
                }
                factory = null;
                throw ex;
            }
            catch (TimeoutException ex)
            {
                if (null != channel)
                    channel.Abort();
                throw ex;
            }
            catch (System.Exception ex)
            {
                if (null != channel)
                    channel.Close();
                throw ex;
            }
            return result;
        }       

        #endregion        

        #region IDataPortalServer Members

        public DataPortalResult Create(System.Type objectType, object criteria, DataPortalContext context)
        {
            CriteriaRequest cr = new CriteriaRequest()
            {
                TypeName = objectType.AssemblyQualifiedName,
                CriteriaData = criteria as Criteria,//MobileFormatter.Serialize(criteria),
                Context = context
            };
            WcfResponse response = this.Invoke<IWcfPortal, WcfResponse>(op => op.Create(cr), objectType.Module.Name);
            if (response.ErrorData != null)
            {
                Exception ex = ServiceExceptionDetail.GetException(response.ErrorData);
                throw new Server.DataPortalException(ex.Message, ex);
            }
            object result = response.ObjectData;
            if (result is Exception)
                throw (Exception)result;
            return (DataPortalResult)result;
        }

        public DataPortalResult Fetch(System.Type objectType, object criteria, DataPortalContext context)
        {
            CriteriaRequest cr = new CriteriaRequest()
            {
                TypeName = objectType.AssemblyQualifiedName,
                CriteriaData = criteria as Criteria,//MobileFormatter.Serialize(criteria),
                Context = context
            };
            WcfResponse response = Invoke<IWcfPortal, WcfResponse>(op => op.Fetch(cr), objectType.Module.Name);
            //对Wcf的错误进行封装，使之返回原始错误
            if (response.ErrorData != null)
            {
                Exception ex = ServiceExceptionDetail.GetException(response.ErrorData);
                throw new Server.DataPortalException(ex.Message, ex);
            }
            object result = response.ObjectData;
            if (result is Exception)
                throw (Exception)result;
            return (DataPortalResult)result;
        }

        public DataPortalResult Update(object obj, DataPortalContext context)
        {
            UpdateRequest ur = new UpdateRequest()
            {
                ObjectData = obj,
                Context = context
            };
            WcfResponse response = Invoke<IWcfPortal, WcfResponse>(op => op.Update(ur), obj.GetType().Module.Name);
            if (response.ErrorData != null)
            {
                Exception ex = ServiceExceptionDetail.GetException(response.ErrorData);
                throw new Server.DataPortalException(ex.Message, ex);
            }
            object result = response.ObjectData;
            if (result is Exception)
                throw (Exception)result;
            return (DataPortalResult)result;
        }

        public DataPortalResult Delete(System.Type objectType, object criteria, DataPortalContext context)
        {
            CriteriaRequest cr = new CriteriaRequest()
            {
                TypeName = objectType.AssemblyQualifiedName,
                CriteriaData = criteria as Criteria,//MobileFormatter.Serialize(criteria),
                Context = context
            };
            WcfResponse response = Invoke<IWcfPortal, WcfResponse>(op => op.Delete(cr), objectType.Module.Name);
            if (response.ErrorData != null)
            {
                Exception ex = ServiceExceptionDetail.GetException(response.ErrorData);
                throw new Server.DataPortalException(ex.Message, ex);
            }
            object result = response.ObjectData;
            if (result is Exception)
                throw (Exception)result;
            return (DataPortalResult)result;
        }

        #endregion
    }
}
