﻿using System;
using Wcdf.Core;
using Wcdf.Serialization;
using Wcdf.Serialization.Mobile;

namespace Wcdf.Rules
{
    /// <summary>
    /// 保存不符合业务规则的细则.
    /// Stores details about a specific broken business rule.
    /// </summary>
    [Serializable]
    public partial class BrokenRule : MobileObject
    {
#if SILVERLIGHT
    /// <summary>
    /// Creates an instance of this object.
    /// </summary>
    public BrokenRule()
    { }
#else
        internal BrokenRule()
        { }
#endif

        private string _ruleName;
        private string _description;
        private string _property;
        private RuleSeverity _severity;
        private string _originProperty;

        /// <summary>
        /// 规则名称
        /// </summary>
        /// <value>规则名称.</value>
        public string RuleName
        {
            get { return _ruleName; }
            internal set { _ruleName = value; }
        }

        /// <summary>
        /// 破坏规则的描述的访问
        /// </summary>
        /// <value>规则描述.</value>
        public string Description
        {
            get { return _description; }
            internal set { _description = value; }
        }

        /// <summary>
        /// 被规则所影响的属性
        /// </summary>
        /// <value>被规则所影响的属性.</value>
        public string Property
        {
            get { return _property; }
            internal set { _property = value; }
        }

        /// <summary>
        /// 规则严重性.
        /// </summary>
        /// <value></value>
        /// <returns></returns>
        /// <remarks></remarks>
        public RuleSeverity Severity
        {
            get { return _severity; }
            internal set { _severity = value; }
        }

        /// <summary>
        /// Gets or sets the origin property.
        /// </summary>
        /// <value>The origin property.</value>
        public string OriginProperty
        {
            get { return _originProperty; }
            internal set { _originProperty = value; }
        }

        #region MobileObject overrides

        /// <summary>
        /// Override this method to insert your field values
        /// into the MobileFormatter serialzation stream.
        /// </summary>
        /// <param name="info">
        /// Object containing the data to serialize.
        /// </param>
        /// <param name="mode">
        /// The StateMode indicating why this method was invoked.
        /// </param>
        protected override void OnGetState(SerializationInfo info, StateMode mode)
        {
            info.AddValue("_ruleName", _ruleName);
            info.AddValue("_description", _description);
            info.AddValue("_property", _property);
            info.AddValue("_severity", (int)_severity);
            info.AddValue("_originProperty", _originProperty);

            base.OnGetState(info, mode);
        }

        /// <summary>
        /// Override this method to retrieve your field values
        /// from the MobileFormatter serialzation stream.
        /// </summary>
        /// <param name="info">
        /// Object containing the data to serialize.
        /// </param>
        /// <param name="mode">
        /// The StateMode indicating why this method was invoked.
        /// </param>
        protected override void OnSetState(SerializationInfo info, StateMode mode)
        {
            _ruleName = info.GetValue<string>("_ruleName");
            _description = info.GetValue<string>("_description");
            _property = info.GetValue<string>("_property");
            _severity = info.GetValue<RuleSeverity>("_severity");
            _originProperty = info.GetValue<string>("_originProperty");

            base.OnSetState(info, mode);
        }

        #endregion
    }
}
