﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Wcdf.Rules
{
    /// <summary>
    /// 包含规则结果
    /// </summary>
    public class RuleResult
    {
        /// <summary>
        /// 取得创建规则结果的规则名称.        
        /// </summary>
        public string RuleName { get; private set; }
        /// <summary>
        /// 该规则是否通过
        /// </summary>
        public bool Success { get; private set; }
        /// <summary>
        /// 规则失败的可读性描述.面向用户        
        /// </summary>
        public string Description { get; private set; }
        /// <summary>
        /// 规则失败类型
        /// </summary>
        public RuleSeverity Severity { get; set; }
        /// <summary>
        /// 中止处理.异步时应用        
        /// </summary>
        public bool StopProcessing { get; set; }
        /// <summary>
        /// 规则的首要属性
        /// </summary>
        public Wcdf.Core.IPropertyInfo PrimaryProperty { get; private set; }
        /// <summary>
        /// 规则影响的属性列表,可通过属性更改事情传达给UI通知.

        /// </summary>
        public List<Core.IPropertyInfo> Properties { get; set; }
        /// <summary>
        /// 新的属性值字典.规则完成时.用于更新业务对象的属性        
        /// </summary>
        public Dictionary<Core.IPropertyInfo, object> OutputPropertyValues { get; set; }

        /// <summary>
        /// Creates a successful result.
        /// </summary>
        /// <param name="ruleName">Unique name of the rule creating
        /// this result.</param>
        /// <param name="property">Property to which this result should
        /// be attached.</param>
        public RuleResult(string ruleName, Core.IPropertyInfo property)
        {
            RuleName = ruleName;
            PrimaryProperty = property;
            Success = true;
        }

        /// <summary>
        /// Creates a failure result.
        /// </summary>
        /// <param name="ruleName">Unique name of the rule creating
        /// this result.</param>
        /// <param name="property">Property to which this result should
        /// be attached.</param>
        /// <param name="description">可读的人性化规则失败描述.Null或空时表示规则通过.</param>
        public RuleResult(string ruleName, Core.IPropertyInfo property, string description)
        {
            RuleName = ruleName;
            PrimaryProperty = property;
            Description = description;
            Success = string.IsNullOrEmpty(description);
            if (!Success)
                Severity = RuleSeverity.Error;
        }
    }
}
