﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Principal;
using Wcdf.Serialization.Mobile;

namespace Wcdf.Security
{
    /// <summary>
    /// 自定义角色对象需要继承的类.    
    /// </summary>
    [Serializable()]
    public class BusinessPrincipalBase : IPrincipal
    {
        private IIdentity _identity;

        /// <summary>
        /// Creates an instance of the object.
        /// </summary>
        protected BusinessPrincipalBase() { _identity = new UnauthenticatedIdentity(); }

        /// <summary>
        /// Returns the user's identity object.
        /// </summary>
        public virtual IIdentity Identity
        {
            get { return _identity; }
            protected set { _identity = value; }
        }

        /// <summary>
        /// Returns a value indicating whether the
        /// user is in a given role.
        /// </summary>
        /// <param name="role">Name of the role.</param>
        public virtual bool IsInRole(string role)
        {
            var check = _identity as ICheckRoles;
            if (check != null)
                return check.IsInRole(role);
            else
                return false;
        }

        /// <summary>
        /// Creates an instance of the object.
        /// </summary>
        /// <param name="identity">Identity object for the user.</param>
        protected BusinessPrincipalBase(IIdentity identity)
        {
            _identity = identity;
        }
    }
}
