﻿
namespace Wcdf.Security
{
    /// <summary>
    /// 定义当前用户是否有属性及方法执行,读取.写入权限.    
    /// </summary>
    public interface IAuthorizeReadWrite
    {
        /// <summary>
        /// 返回<see langword="true" />允许写属性        
        /// </summary>
        /// <returns><see langword="true" /> if write is allowed.</returns>
        /// <param name="propertyName">Name of the property to read.</param>
        bool CanWriteProperty(string propertyName);
        /// <summary>
        /// Returns <see langword="true" /> if the user is allowed to read the
        /// specified property.
        /// </summary>
        /// <returns><see langword="true" /> if read is allowed.</returns>
        /// <param name="propertyName">Name of the property to read.</param>
        bool CanReadProperty(string propertyName);
        /// <summary>
        /// Returns <see langword="true" /> if the user is allowed to execute 
        /// the specified method.
        /// </summary>
        /// <returns><see langword="true" /> if execute is allowed.</returns>
        /// <param name="methodName">Name of the method to execute.</param>
        bool CanExecuteMethod(string methodName);
    }
}
