﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Wcdf.Serialization.Mobile
{
    /// <summary>
    /// Interface to be implemented by any object
    /// that supports serialization by the
    /// MobileFormatter.
    /// </summary>
    public interface IMobileObject
    {
        /// <summary>
        /// 序列化数据到SerializationInfo参数中        
        /// </summary>
        /// <param name="info">
        /// Object to contain the serialized data.
        /// </param>
        void GetState(SerializationInfo info);

        /// <summary>
        /// Method called by MobileFormatter when an object
        /// should serialize its child references. The data should be
        /// serialized into the SerializationInfo parameter.
        /// </summary>
        /// <param name="info">
        /// Object to contain the serialized data.
        /// </param>
        /// <param name="formatter">
        /// Reference to the formatter performing the serialization.
        /// </param>
        void GetChildren(SerializationInfo info, MobileFormatter formatter);

        /// <summary>
        /// 从SerializationInfo参数中反序列化数据.
        /// </summary>
        /// <param name="info">
        /// Object containing the serialized data.
        /// </param>
        void SetState(SerializationInfo info);

        /// <summary>
        /// Method called by MobileFormatter when an object
        /// should deserialize its child references. The data should be
        /// deserialized from the SerializationInfo parameter.
        /// </summary>
        /// <param name="info">
        /// Object containing the serialized data.
        /// </param>
        /// <param name="formatter">
        /// Reference to the formatter performing the deserialization.
        /// </param>
        void SetChildren(SerializationInfo info, MobileFormatter formatter);
    }
}
