﻿using System.ServiceModel.Dispatcher;
using Wcdf.Security;
using Wcdf.Rules;
using System;

namespace Wcdf.Server.Hosts.WcfChannel
{
    /// <summary>
    /// 服务端消息调度检查,
    /// </summary>
    public class ServiceDispatchMessageInspector : IDispatchMessageInspector
    {
        #region IDispatchMessageInspector 成员

        public object AfterReceiveRequest(ref System.ServiceModel.Channels.Message request, System.ServiceModel.IClientChannel channel, System.ServiceModel.InstanceContext instanceContext)
        {
            int i = request.Headers.FindHeader(DataPortalContext.Name, DataPortalContext.NameSpace);
            if (i < 0)
            {
                //没有找到，外部需求进行验证,方式待后定
                //DataPermissionManager.Instance.Validate(context.UserName, context.Password);
            }
            var context = request.Headers.GetHeader<DataPortalContext>(i);
            if (context != null)
            {
                if (context.Principal is Wcdf.Security.UnauthenticatedPrincipal)
                {
                    //ApplicationContext.ClientContext["LoginAuth"] = "true";
                    var id = context.Principal.Identity as UnauthenticatedIdentity;
                    DataPermissionManager.Instance.Validate(id.UserName, id.Password);
                }
                else if (ApplicationContext.ValidateUserNameType != "FirstLogin")
                {
                    var id = context.Principal.Identity as Wcdf.Security.WcdfIdentity;
                    DataPermissionManager.Instance.Validate(id.UserInfo.UserName, id.UserInfo.Password);
                }
                SetContext(context);
            }
            return null;
        }

        public void BeforeSendReply(ref System.ServiceModel.Channels.Message reply, object correlationState)
        {
            ClearContext();
        }

        #endregion

        private void SetContext(DataPortalContext context)
        {
            ApplicationContext.SetLogicalExecutionLocation(ApplicationContext.LogicalExecutionLocations.Server);
            // set the context value so everyone knows the
            // code is running on the server
            ApplicationContext.SetExecutionLocation(ApplicationContext.ExecutionLocations.Server);

            // set the app context to the value we got from the
            // client
            ApplicationContext.SetContext(context.ClientContext, context.GlobalContext);

            // set the thread's culture to match the client
            System.Threading.Thread.CurrentThread.CurrentCulture =
              new System.Globalization.CultureInfo(context.ClientCulture);
            System.Threading.Thread.CurrentThread.CurrentUICulture =
              new System.Globalization.CultureInfo(context.ClientUICulture);

            if (ApplicationContext.AuthenticationType == "Windows")
            {
                // 当采用集成身份验证时，客户端传递来凭据的应该NULL
                if (context.Principal != null)
                {
                    System.Security.SecurityException ex =
                      new System.Security.SecurityException(Wcdf.Properties.Resources.NoPrincipalAllowedException);
                    //ex.Action = System.Security.Permissions.SecurityAction.Deny;
                    throw ex;
                }
                // 取得服务端本机的验证凭据
                AppDomain.CurrentDomain.SetPrincipalPolicy(System.Security.Principal.PrincipalPolicy.WindowsPrincipal);
            }
            else
            {
                // 当客户端采用其他认证时，需要上传客户端凭据
                if (context.Principal == null)
                {
                    System.Security.SecurityException ex =
                      new System.Security.SecurityException(
                        Wcdf.Properties.Resources.BusinessPrincipalException + " Nothing");
                    //ex.Action = System.Security.Permissions.SecurityAction.Deny;
                    throw ex;
                }
                //以客户端凭据当本服务端凭据，以传递到下一个服务端
                ApplicationContext.User = context.Principal;
            }
        }

        private void ClearContext()
        {
            ApplicationContext.Clear();
            if (ApplicationContext.AuthenticationType != "Windows")
                ApplicationContext.User = null;
        }
    }
}
