﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Activation;
using Wcdf.Server.Hosts.WcfChannel;
using Wcdf.Serialization.Mobile;

namespace Wcdf.Server.Hosts
{
    /// <summary>
    /// Exposes server-side DataPortal functionality
    /// through WCF.
    /// </summary>
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall)]
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class WcfPortal : IWcfPortal
    {
        #region IWcfPortal Members

        /// <summary>
        /// Create a new business object.
        /// </summary>
        /// <param name="request">The request parameter object.</param>
        [OperationBehavior(Impersonation = ImpersonationOption.Allowed)]
        public WcfResponse Create(CriteriaRequest request)
        {
            object criteria = request.CriteriaData;//GetCriteria(request.CriteriaData);
            Type bizType = Type.GetType(request.TypeName);
            DataPortalContext context = request.Context;//MobileFormatter.Deserialize(request.PortalContextData) as DataPortalContext;
            DataPortal portal = new DataPortal();
            var result = new WcfResponse();
            try
            {
                result.ObjectData = portal.Create(bizType, criteria, context);
            }
            catch (Exception ex)
            {
                result.ErrorData = new ServiceExceptionDetail(ex);
            }
            return result;
        }

        /// <summary>
        /// Get an existing business object.
        /// </summary>
        /// <param name="request">The request parameter object.</param>
        [OperationBehavior(Impersonation = ImpersonationOption.Allowed)]
        public WcfResponse Fetch(CriteriaRequest request)
        {
            object criteria = request.CriteriaData;//GetCriteria(request.CriteriaData);
            Type bizType = Type.GetType(request.TypeName);
            DataPortalContext context = request.Context;//MobileFormatter.Deserialize(request.PortalContextData) as DataPortalContext;
            DataPortal portal = new DataPortal();
            var result = new WcfResponse();
            try
            {
                result.ObjectData = portal.Fetch(bizType, criteria, context);
            }
            catch (Exception ex)
            {
                result.ErrorData = new ServiceExceptionDetail(ex);
            }
            return result;
        }

        /// <summary>
        /// Update a business object.
        /// </summary>
        /// <param name="request">The request parameter object.</param>
        [OperationBehavior(Impersonation = ImpersonationOption.Allowed)]
        public WcfResponse Update(UpdateRequest request)
        {
            object obj = request.ObjectData;
            DataPortalContext context = request.Context;//MobileFormatter.Deserialize(request.PortalContextData) as DataPortalContext;
            DataPortal portal = new DataPortal();
            var result = new WcfResponse();
            try
            {
                result.ObjectData = portal.Update(obj, context);
            }
            catch (Exception ex)
            {
                result.ErrorData = new ServiceExceptionDetail(ex);
            }
            return result;
        }

        /// <summary>
        /// Delete a business object.
        /// </summary>
        /// <param name="request">The request parameter object.</param>
        [OperationBehavior(Impersonation = ImpersonationOption.Allowed)]
        public WcfResponse Delete(CriteriaRequest request)
        {
            object criteria = request.CriteriaData;//GetCriteria(request.CriteriaData);
            Type bizType = Type.GetType(request.TypeName);
            DataPortalContext context = request.Context;//MobileFormatter.Deserialize(request.PortalContextData) as DataPortalContext;
            DataPortal portal = new DataPortal();
            var result = new WcfResponse();
            try
            {
                result.ObjectData = MobileFormatter.Serialize(portal.Delete(bizType, criteria, context));
            }
            catch (Exception ex)
            {
                result.ErrorData = new ServiceExceptionDetail(ex);
            }
            return result;
        }

        #endregion

        #region Criteria
        /// <summary>
        /// 反序列化Criteria对象
        /// </summary>
        /// <param name="criteriaData"></param>
        /// <returns></returns>
        private static object GetCriteria(byte[] criteriaData)
        {
            object criteria = null;
            if (criteriaData != null)
                criteria = MobileFormatter.Deserialize(criteriaData);
            return criteria;
        }

        #endregion
    }
}
