﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Wcdf.Server
{
    /// <summary>
    /// 服务端数据门户需要继承的接口.    
    /// </summary>
    public interface IDataPortalServer
    {
        /// <summary>
        /// 创建新的业务对象
        /// </summary>
        /// <param name="objectType">需要创建的业务对象.</param>
        /// <param name="criteria">创建参数.</param>
        /// <param name="context">
        /// <see cref="Server.DataPortalContext" />传递到服务器的上下文信息.
        /// </param>
        DataPortalResult Create(Type objectType, object criteria, DataPortalContext context);
        /// <summary>
        /// 取得业务对象.
        /// </summary>
        /// <param name="objectType">指定的业务对象类型.</param>
        /// <param name="criteria">过滤器.</param>
        /// <param name="context">
        /// <see cref="Server.DataPortalContext" />传递到服务器的上下文信息.
        /// </param>
        DataPortalResult Fetch(Type objectType, object criteria, DataPortalContext context);
        /// <summary>
        /// 更新业务对象.
        /// </summary>
        /// <param name="obj">需要更新的业务对象实例.</param>
        /// <param name="context">
        /// <see cref="Server.DataPortalContext" />传递到服务器的上下文信息.
        /// </param>
        DataPortalResult Update(object obj, DataPortalContext context);
        /// <summary>
        /// 删除业务对象
        /// </summary>
        /// <param name="objectType">指定的业务对象类型.</param>
        /// <param name="criteria">过滤器.</param>
        /// <param name="context">
        /// <see cref="Server.DataPortalContext" />传递到服务器的上下文信息.
        /// </param>
        DataPortalResult Delete(Type objectType, object criteria, DataPortalContext context);
    }
}
