﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Wcdf.Reflection;

namespace Wcdf.Server
{
    /// <summary>    
    /// 服务端绑定业务对象,通过该类对业务对象方法及服务端方法进行调用.
    /// </summary>
    public class ServiceBoundObject
    {
        /// <summary>
        /// 业务对象实例.
        /// </summary>
        public object Instance { get; private set; }

        private IServiceObject _serviceObject;
        private IServiceObject ServiceObject
        {
            get
            {
                if (_serviceObject == null)
                {
                    _serviceObject = new DefaultServiceObject();
                }
                return _serviceObject;
            }
            set
            {
                _serviceObject = value;
            }
        }
        /// <summary>
        /// 创建指定类型的服务端处理类并创建业务实例        
        /// </summary>
        /// <param name="objectType">
        /// 需要处理的对象类型.
        /// </param>
        /// <remarks>
        /// 应用本构造方法时.需要注意.objectType对象实例并不在本类中创建.
        /// 请应用SetInstance(object instance)方法传入对象实例
        /// </remarks>
        public ServiceBoundObject(Type objectType)
        {
            this._serviceObject = ServiceObjectFactory.Instance.GetServiceObject(objectType) as IServiceObject;
            this.Instance = MethodCaller.CreateInstance(objectType);
        }

        /// <summary>
        /// 传入指定的待处理实例,并设置相应的服务端处理对象.
        /// </summary>
        /// <param name="instance"></param>
        public ServiceBoundObject(object instance)
        {
            SetInstance(instance);
        }


        /// <summary>
        /// 设置业务对象实例.不改变服务对象
        /// </summary>
        /// <param name="instance"></param>
        public void SetInstance(object instance)
        {
            //类型相同的话,不需要重设业务对象
            if (this.Instance == null || !this.Instance.GetType().Equals(instance.GetType()))
            {
                this._serviceObject = ServiceObjectFactory.Instance.GetServiceObject(instance.GetType()) as IServiceObject;
            }
            this.Instance = instance;
        }

        /// <summary>
        /// 使用反射动态调用实例的含参数方法        
        /// </summary>
        /// <param name="method">
        /// 方法名.
        /// </param>
        /// <param name="parameters">
        /// 与方法参数对应的参数列表.
        /// </param>
        public object CallMethodIfImplemented(string method, params object[] parameters)
        {
            return MethodCaller.CallMethodIfImplemented(this.Instance, method, parameters);
        }

        /// <summary>
        /// 使用反射动态调用实例的方法.
        /// </summary>
        /// <param name="method">
        /// 方法名.
        /// </param>
        public object CallMethod(string method)
        {
            return MethodCaller.CallMethod(this.Instance, method);
        }

        /// <summary>
        /// 使用反射动态调用实例的方法.
        /// </summary>
        /// <param name="method">
        /// 方法名.
        /// </param>
        /// <param name="parameters">
        /// 与方法参数对应的参数列表.
        /// </param>
        public object CallMethod(string method, params object[] parameters)
        {
            return MethodCaller.CallMethod(this.Instance, method, parameters);
        }

        /// <summary>
        /// 调用服务方法前
        /// </summary>
        /// <param name="args"></param>
        public void OnDataPorta_Invoke(DataPortalEventArgs args)
        {
            this.ServiceObject.OnDataPorta_Invoke(this.Instance,args);
        }

        public void OnDataPorta_InvokeComplete(DataPortalEventArgs args)
        {
            this.ServiceObject.OnDataPorta_InvokeComplete(this.Instance,args);
        }
    }
}
