﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace Wcdf.Server
{
    /// <summary>
    /// 业务对象到服务端后对应的服务对象工作
    /// </summary>
    public sealed class ServiceObjectFactory
    {
        public readonly static ServiceObjectFactory Instance = new ServiceObjectFactory();

        private ServiceObjectFactory()
        {
            LoadConfig();
        }

        private Dictionary<string, string> _config = new Dictionary<string, string>();
        //private static Dictionary<string, Type> _soCache = new Dictionary<string, Type>();

        //private static Dictionary<string, IServiceBusinessObject> FactoryCache = new Dictionary<string, IServiceBusinessObject>();        
        /// <summary>
        /// 将配置的类加入到缓存中.该方法为外部方法调用.
        /// </summary>
        /// <param name="bizType"></param>
        /// <param name="SvrType"></param>
        public void AddConfig(string bizType, string SvrType)
        {
            _config.Add(bizType, SvrType);
        }

        public string GetServiceType(Type bizType)
        {
            if (_config.ContainsKey(bizType.FullName))
                return _config[bizType.FullName];
            else
                return string.Empty;
        }

        public object GetServiceObject(Type bizType)
        {
            string typeName = GetServiceType(bizType);
            if (!string.IsNullOrEmpty(typeName))
                return Wcdf.Reflection.MethodCaller.CreateInstance(Type.GetType(typeName, true, true));
            return new DefaultServiceObject();
        }

        private void LoadConfig()
        {
            string filePath = string.Concat(AppDomain.CurrentDomain.BaseDirectory, @"ServiceConfig\ServiceBound.xml");
            if (!System.IO.File.Exists(filePath))
            {
                //throw new System.IO.FileNotFoundException("程序文件缺失:LookUp.Config配置文件不存在!");
                return;
            }
            //解释配置文件
            XElement appServer = XElement.Load(filePath);
            var configQuery = from configs in appServer.Elements("Bound")
                              select configs;
            foreach (XElement ele in configQuery)
            {
                if (ele.Attribute("BizType") == null || ele.Attribute("SvrType") == null)
                    continue;
                var bt = ele.Attribute("BizType").Value;
                var st = ele.Attribute("SvrType").Value;
                AddConfig(bt, st);                
            }
            
        }
    }
}
