﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Wcdf.Core;
using System.Collections.ObjectModel;
using System.ComponentModel;

namespace Wcdf.ToolClass
{
    public class EntityTree<T, C> : INotifyPropertyChanged
        where C : BusinessBase
        where T : ObservableBindingList<C>
    {
        private C _node;
        public C Node
        {
            get
            {
                return _node;
            }
            set
            {
                _node = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("Node"));
                }
            }
        }
        public ObservableCollection<EntityTree<T, C>> _childNodes;
        public ObservableCollection<EntityTree<T, C>> ChildNodes
        {
            get { return _childNodes; }
            set
            {
                _childNodes = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("ChildNodes"));
                }
            }
        }

        public EntityTree<T, C> Parent { get; set; }

        private IPropertyInfo childInfo;
        private IPropertyInfo parentInfo;

        public EntityTree()
        {
        }

        public EntityTree(T dataSource, C child, string childName, string parentName)
        {
            var PropertyInfos = Wcdf.Core.FieldManager.PropertyInfoManager.GetRegisteredProperties(typeof(C));

            childInfo = PropertyInfos.Find(o => o.FriendlyName == childName);//child.GetFieldManager().FindProperty(childName);
            parentInfo = PropertyInfos.Find(o => o.FriendlyName == parentName);//child.GetFieldManager().FindProperty(parentName);
            if (childInfo == null || parentInfo == null)
                throw new Exception(Wcdf.Properties.Resources.PropertyGetNotAllowed);
            var query = dataSource.Where(o => (o.GetFieldManager().GetFieldData(parentInfo).Value==null ? string.Empty : o.GetFieldManager().GetFieldData(parentInfo).Value.ToString()) == string.Empty);
            if (query.Count() == 0)
                return;
            this.ChildNodes = new ObservableCollection<EntityTree<T, C>>();
            foreach (var item in query)
            {
                var childItem = new EntityTree<T, C>();

                childItem.Node = item;
                GetChildNodes(childItem, dataSource);
                this.AddNewItem(childItem);
                //childItem.Parent = this;
                //this.ChildNodes.Add(childItem);
            }
        }

        public void GetChildNodes(EntityTree<T, C> parent, T dataSource)
        {
            //List<SysPermission> rav = new List<SysPermission>();
            var query = dataSource.Where(o => (o.GetFieldManager().GetFieldData(parentInfo).Value == null ? string.Empty : o.GetFieldManager().GetFieldData(parentInfo).Value.ToString())
                == (parent.Node.GetFieldManager().GetFieldData(childInfo).Value == null ? string.Empty : parent.Node.GetFieldManager().GetFieldData(childInfo).Value.ToString()));
            if (query.Count() == 0)
                return;
            //parent.ChildNodes = new ObservableCollection<EntityTree<T, C>>();
            foreach (var item in query)
            {
                var pt = new EntityTree<T, C>();
                pt.Node = item;
                GetChildNodes(pt, dataSource);
                parent.AddNewItem(pt);
                //pt.Parent = parent;
                //parent.ChildNodes.Add(pt);
            }
        }

        public void AddNewItem(EntityTree<T, C> item)
        {
            if (this.ChildNodes == null)
                this.ChildNodes = new ObservableCollection<EntityTree<T, C>>();
            item.Parent = this;
            this.ChildNodes.Add(item);
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}
